@@ -28,12 +28,13 @@
 #include "dnscache.h"
 #include "alarms.h"
 #include "stats/stats-registry.h"
+#include "metrics/metrics.h"
 #include "healthcheck/healthcheck-stats.h"
 #include "logmsg/logmsg.h"
 #include "logsource.h"
 #include "logwriter.h"
 #include "afinter.h"
-#include "template/templates.h"
+#include "template/globals.h"
 #include "hostname.h"
 #include "mainloop-call.h"
 #include "service-management.h"
@@ -48,6 +49,7 @@
 #include "msg-stats.h"
 #include "timeutils/cache.h"
 #include "multi-line/multi-line-factory.h"
+#include "filterx/filterx-globals.h"
 
 #include <iv.h>
 #include <iv_work.h>
@@ -226,10 +228,10 @@ app_startup(void)
   alarm_init();
   main_loop_thread_resource_init();
   stats_init();
+  metrics_global_init();
   healthcheck_stats_global_init();
   tzset();
   log_msg_global_init();
-  log_tags_global_init();
   log_source_global_init();
   log_template_global_init();
   value_pairs_global_init();
@@ -242,6 +244,7 @@ app_startup(void)
   msg_stats_init();
   timeutils_global_init();
   multi_line_global_init();
+  filterx_global_init();
 }
 
 void
@@ -269,6 +272,7 @@ app_shutdown(void)
   msg_stats_deinit();
   run_application_hook(AH_SHUTDOWN);
 
+  filterx_global_deinit();
   multi_line_global_deinit();
   main_loop_thread_resource_deinit();
   secret_storage_deinit();
@@ -276,10 +280,10 @@ app_shutdown(void)
   scratch_buffers_global_deinit();
   value_pairs_global_deinit();
   log_template_global_deinit();
-  log_tags_global_deinit();
   log_msg_global_deinit();
 
   afinter_global_deinit();
+  metrics_global_deinit();
   stats_destroy();
   child_manager_deinit();
   g_list_foreach(application_hooks, (GFunc) g_free, NULL);
