@@ -61,7 +61,7 @@ _resolve_unknown_blockargs_as_varargs(gpointer key, gpointer value, gpointer use
 
   if (!defaults || !cfg_args_contains(defaults, key))
     {
-      g_string_append_printf(varargs, "%s(%s) ", (gchar *)key, (gchar *)value);
+      g_string_append_printf(varargs, "%s(%s) ", (gchar *)key, (gchar *)value ? : "");
     }
 }
 
