@@ -29,6 +29,7 @@
 #include "logpipe.h"
 #include "logqueue.h"
 #include "cfg.h"
+#include "signal-slot-connector/signal-slot-connector.h"
 
 /*
  * Drivers overview
@@ -70,6 +71,7 @@ typedef struct _LogDriverPlugin LogDriverPlugin;
 
 struct _LogDriverPlugin
 {
+  SignalSlotConnector *signal_connector;
   const gchar *name;
   /* this function is called when the plugin is attached to a LogDriver
    * instance.  It should do whatever it is necessary to extend the
@@ -159,7 +161,7 @@ struct _LogDestDriver
   LogDriver super;
 
   LogQueue *(*acquire_queue)(LogDestDriver *s, const gchar *persist_name, gint stats_level,
-                             const StatsClusterKeyBuilder *driver_sck_builder,
+                             StatsClusterKeyBuilder *driver_sck_builder,
                              StatsClusterKeyBuilder *queue_sck_builder);
   void (*release_queue)(LogDestDriver *s, LogQueue *q);
 
@@ -175,7 +177,7 @@ struct _LogDestDriver
 /* returns a reference */
 static inline LogQueue *
 log_dest_driver_acquire_queue(LogDestDriver *self, const gchar *persist_name, gint stats_level,
-                              const StatsClusterKeyBuilder *driver_sck_builder,
+                              StatsClusterKeyBuilder *driver_sck_builder,
                               StatsClusterKeyBuilder *queue_sck_builder)
 {
   LogQueue *q;
