@@ -58,7 +58,6 @@ log_filter_pipe_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_op
 {
   LogFilterPipe *self = (LogFilterPipe *) s;
   gboolean res;
-  gchar *filter_result;
 
   msg_trace(">>>>>> filter rule evaluation begin",
             evt_tag_str("rule", self->name),
@@ -67,25 +66,24 @@ log_filter_pipe_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_op
 
   res = filter_expr_eval_root(self->expr, &msg, path_options);
 
+  msg_trace("<<<<<< filter rule evaluation result",
+            evt_tag_str("result", res ? "matched" : "unmatched"),
+            evt_tag_str("rule", self->name),
+            log_pipe_location_tag(s),
+            evt_tag_msg_reference(msg));
+
   if (res)
     {
-      filter_result = "MATCH - Forwarding message to the next LogPipe";
       log_pipe_forward_msg(s, msg, path_options);
       stats_counter_inc(self->matched);
     }
   else
     {
-      filter_result = "UNMATCHED - Dropping message from LogPipe";
       if (path_options->matched)
         (*path_options->matched) = FALSE;
       log_msg_drop(msg, path_options, AT_PROCESSED);
       stats_counter_inc(self->not_matched);
     }
-  msg_trace("<<<<<< filter rule evaluation result",
-            evt_tag_str("result", filter_result),
-            evt_tag_str("rule", self->name),
-            log_pipe_location_tag(s),
-            evt_tag_msg_reference(msg));
 }
 
 static LogPipe *
@@ -124,7 +122,7 @@ log_filter_pipe_new(FilterExprNode *expr, GlobalConfig *cfg)
   LogFilterPipe *self = g_new0(LogFilterPipe, 1);
 
   log_pipe_init_instance(&self->super, cfg);
-  self->super.flags |= PIF_CONFIG_RELATED;
+  self->super.flags |= PIF_CONFIG_RELATED + PIF_SYNC_FILTERX;
   self->super.init = log_filter_pipe_init;
   self->super.queue = log_filter_pipe_queue;
   self->super.free_fn = log_filter_pipe_free;
