@@ -45,17 +45,25 @@ filter_tags_eval(FilterExprNode *s, LogMessage **msgs, gint num_msg, LogTemplate
       LogTagId tag_id = g_array_index(self->tags, LogTagId, i);
       if (log_msg_is_tag_by_id(msg, tag_id))
         {
-          res = TRUE;
-          msg_trace("tags() evaluation started",
+          msg_trace("tags() evaluation result, matching tag is found",
                     evt_tag_str("tag", log_tags_get_by_id(tag_id)),
                     evt_tag_msg_reference(msg));
+
+          res = TRUE;
           return res ^ s->comp;
         }
+      else
+        {
+          msg_trace("tags() evaluation progress, tag is not set",
+                    evt_tag_str("tag", log_tags_get_by_id(tag_id)),
+                    evt_tag_int("value", log_msg_is_tag_by_id(msg, tag_id)),
+                    evt_tag_msg_reference(msg));
+        }
     }
 
-  res = FALSE;
-  msg_trace("tags() evaluation started",
+  msg_trace("tags() evaluation result, none of the tags is present",
             evt_tag_msg_reference(msg));
+  res = FALSE;
   return res ^ s->comp;
 }
 
