@@ -27,6 +27,7 @@
 
 #include "syslog-ng.h"
 #include "logmsg/logmsg.h"
+#include "filterx/filterx-eval.h"
 #include "cfg.h"
 #include "atomic.h"
 #include "messages.h"
@@ -41,28 +42,42 @@
 #define NC_REOPEN_REQUIRED 6
 #define NC_FILE_DELETED 7
 
+/* notify result mask values */
+#define NR_OK          0x0000
+#define NR_STOP_ON_EOF 0x0001
+
 /* indicates that the LogPipe was initialized */
 #define PIF_INITIALIZED       0x0001
 /* indicates that this LogPipe got cloned into the tree already */
 #define PIF_INLINED           0x0002
 
-/* log statement flags that are copied to the head of a branch */
-#define PIF_BRANCH_FINAL      0x0004
-#define PIF_BRANCH_FALLBACK   0x0008
-#define PIF_BRANCH_PROPERTIES (PIF_BRANCH_FINAL + PIF_BRANCH_FALLBACK)
-
-#define PIF_DROP_UNMATCHED    0x0010
-
-/* branch starting with this pipe wants hard flow control */
-#define PIF_HARD_FLOW_CONTROL 0x0020
-
 /* this pipe is a source for messages, it is not meant to be used to
  * forward messages, syslog-ng will only use these pipes for the
  * left-hand side of the processing graph, e.g. no other pipes may be
  * sending messages to these pipes and these are expected to generate
  * messages "automatically". */
 
-#define PIF_SOURCE            0x0040
+#define PIF_SOURCE            0x0004
+
+/* log statement flags that are copied to the head of a branch */
+#define PIF_BRANCH_FINAL      0x0008
+#define PIF_BRANCH_FALLBACK   0x0010
+#define PIF_BRANCH_PROPERTIES (PIF_BRANCH_FINAL + PIF_BRANCH_FALLBACK)
+
+/* branch starting with this pipe wants hard flow control */
+#define PIF_HARD_FLOW_CONTROL 0x0020
+
+/* LogPipe right after the filter in an "if (filter)" expression */
+#define PIF_CONDITIONAL_MIDPOINT  0x0040
+
+/* LogPipe as the joining element of a junction */
+#define PIF_JUNCTION_END          0x0080
+
+/* node created directly by the user */
+#define PIF_CONFIG_RELATED    0x0100
+
+/* sync filterx state and message in right before calling queue() */
+#define PIF_SYNC_FILTERX      0x0200
 
 /* private flags range, to be used by other LogPipe instances for their own purposes */
 
@@ -207,10 +222,81 @@ struct _LogPathOptions
   gboolean flow_control_requested;
 
   gboolean *matched;
+  const LogPathOptions *lpo_parent_junction;
+  FilterXEvalContext *filterx_context;
 };
 
-#define LOG_PATH_OPTIONS_INIT { TRUE, FALSE, NULL }
-#define LOG_PATH_OPTIONS_INIT_NOACK { FALSE, FALSE, NULL }
+#define LOG_PATH_OPTIONS_INIT { TRUE, FALSE, NULL, NULL }
+#define LOG_PATH_OPTIONS_INIT_NOACK { FALSE, FALSE, NULL, NULL }
+
+/*
+ * Embed a step in our LogPathOptions chain.
+ */
+static inline LogPathOptions *
+log_path_options_chain(LogPathOptions *local_path_options, const LogPathOptions *lpo_previous_hop)
+{
+  *local_path_options = *lpo_previous_hop;
+  return local_path_options;
+}
+
+/* LogPathOptions are chained up at the start of a junction and teared down
+ * at the end (see log_path_options_pop_junction().
+ *
+ * The "matched" value is kept separate on the parent level and the junction
+ * level.  This way the junction can separately act on matching/non-matching
+ * messages and potentially propagate it to the parent (or not), see
+ * logmpx.c for details.
+ * */
+static inline void
+log_path_options_push_junction(LogPathOptions *local_path_options,
+                               gboolean *matched,
+                               const LogPathOptions *lpo_parent_junction)
+{
+  *local_path_options = *lpo_parent_junction;
+  local_path_options->matched = matched;
+  local_path_options->lpo_parent_junction = lpo_parent_junction;
+}
+
+/* Part of the junction related state needs to be "popped" once the
+ * conditional decision is concluded.  This happens in the `if (filter)`
+ * form, once the filter is evaluated, or at the end of the junction.  This
+ * basically resets the "matched" pointer to that of the parent junction.
+ */
+static inline void
+log_path_options_pop_conditional(LogPathOptions *local_path_options)
+{
+  if (local_path_options->lpo_parent_junction)
+    local_path_options->matched = local_path_options->lpo_parent_junction->matched;
+}
+
+/*
+ * Tear down the embedded junction related state from the LogPathOptions
+ * chain.  This implies log_path_options_pop_conditional() as well, which
+ * will do nothing if there was a conditional midpoint (e.g.  `if
+ * (filter)`).
+ *
+ * NOTE: we need to be optional about ->parent being set, as synthetic
+ * messages (e.g.  the likes emitted by db-parser/grouping-by() may arrive
+ * at the end of a junction without actually crossing the beginning of the
+ * same junction.  But this is ok, in these cases we don't need to propagate
+ * our matched state to anywhere, we can assume that the synthetic message
+ * will just follow the same route as the one it was created from.
+ */
+static inline void
+log_path_options_pop_junction(LogPathOptions *local_path_options)
+{
+  log_path_options_pop_conditional(local_path_options);
+
+  if (local_path_options->lpo_parent_junction)
+    local_path_options->lpo_parent_junction = local_path_options->lpo_parent_junction->lpo_parent_junction;
+}
+
+typedef struct _LogPipeOptions LogPipeOptions;
+
+struct _LogPipeOptions
+{
+  gboolean internal;
+};
 
 struct _LogPipe
 {
@@ -226,12 +312,20 @@ struct _LogPipe
   const gchar *persist_name;
   gchar *plugin_name;
   SignalSlotConnector *signal_slot_connector;
+  LogPipeOptions options;
 
   gboolean (*pre_init)(LogPipe *self);
   gboolean (*init)(LogPipe *self);
   gboolean (*deinit)(LogPipe *self);
   void (*post_deinit)(LogPipe *self);
 
+  gboolean (*pre_config_init)(LogPipe *self);
+  /* this event function is used to perform necessary operation, such as
+   * starting worker thread, and etc. therefore, syslog-ng will terminate if
+   * return value is false.
+   */
+  gboolean (*post_config_init)(LogPipe *self);
+
   const gchar *(*generate_persist_name)(const LogPipe *self);
   GList *(*arcs)(LogPipe *self);
 
@@ -242,7 +336,7 @@ struct _LogPipe
   LogPipe *(*clone)(LogPipe *self);
 
   void (*free_fn)(LogPipe *self);
-  void (*notify)(LogPipe *self, gint notify_code, gpointer user_data);
+  gint (*notify)(LogPipe *self, gint notify_code, gpointer user_data);
   GList *info;
 };
 
@@ -259,9 +353,10 @@ G_STATIC_ASSERT(G_STRUCT_OFFSET(LogPipe, queue) - G_STRUCT_OFFSET(LogPipe, flags
 extern gboolean (*pipe_single_step_hook)(LogPipe *pipe, LogMessage *msg, const LogPathOptions *path_options);
 
 LogPipe *log_pipe_ref(LogPipe *self);
-void log_pipe_unref(LogPipe *self);
+gboolean log_pipe_unref(LogPipe *self);
 LogPipe *log_pipe_new(GlobalConfig *cfg);
 void log_pipe_init_instance(LogPipe *self, GlobalConfig *cfg);
+void log_pipe_clone_method(LogPipe *dst, const LogPipe *src);
 void log_pipe_forward_notify(LogPipe *self, gint notify_code, gpointer user_data);
 EVTTAG *log_pipe_location_tag(LogPipe *pipe);
 void log_pipe_attach_expr_node(LogPipe *self, LogExprNode *expr_node);
@@ -270,6 +365,7 @@ void log_pipe_detach_expr_node(LogPipe *self);
 static inline GlobalConfig *
 log_pipe_get_config(LogPipe *s)
 {
+  g_assert(s->cfg);
   return s->cfg;
 }
 
@@ -295,6 +391,8 @@ log_pipe_init(LogPipe *s)
       if (!s->init || s->init(s))
         {
           s->flags |= PIF_INITIALIZED;
+          if (s->cfg)
+            cfg_tree_register_initialized_pipe(&s->cfg->tree, s);
           return TRUE;
         }
       return FALSE;
@@ -313,6 +411,8 @@ log_pipe_deinit(LogPipe *s)
 
           if (s->post_deinit)
             s->post_deinit(s);
+          if (s->cfg)
+            cfg_tree_deregister_initialized_pipe(&s->cfg->tree, s);
           return TRUE;
         }
       return FALSE;
@@ -320,6 +420,22 @@ log_pipe_deinit(LogPipe *s)
   return TRUE;
 }
 
+static inline gboolean
+log_pipe_pre_config_init(LogPipe *s)
+{
+  if (s->pre_config_init)
+    return s->pre_config_init(s);
+  return TRUE;
+}
+
+static inline gboolean
+log_pipe_post_config_init(LogPipe *s)
+{
+  if (s->post_config_init)
+    return s->post_config_init(s);
+  return TRUE;
+}
+
 static inline void
 log_pipe_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options);
 
@@ -351,14 +467,25 @@ log_pipe_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
         }
     }
 
-  if (G_UNLIKELY(s->flags & (PIF_HARD_FLOW_CONTROL)))
+  if ((s->flags & PIF_SYNC_FILTERX))
+    filterx_eval_sync_message(path_options->filterx_context, &msg, path_options);
+
+  if (G_UNLIKELY(s->flags & (PIF_HARD_FLOW_CONTROL | PIF_JUNCTION_END | PIF_CONDITIONAL_MIDPOINT)))
     {
-      local_path_options = *path_options;
-
-      local_path_options.flow_control_requested = 1;
-      path_options = &local_path_options;
-
-      msg_trace("Requesting flow control", log_pipe_location_tag(s));
+      path_options = log_path_options_chain(&local_path_options, path_options);
+      if (s->flags & PIF_HARD_FLOW_CONTROL)
+        {
+          local_path_options.flow_control_requested = 1;
+          msg_trace("Requesting flow control", log_pipe_location_tag(s));
+        }
+      if (s->flags & PIF_JUNCTION_END)
+        {
+          log_path_options_pop_junction(&local_path_options);
+        }
+      if (s->flags & PIF_CONDITIONAL_MIDPOINT)
+        {
+          log_path_options_pop_conditional(&local_path_options);
+        }
     }
 
   if (s->queue)
@@ -370,10 +497,6 @@ log_pipe_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
       log_pipe_forward_msg(s, msg, path_options);
     }
 
-  if (path_options->matched && !(*path_options->matched) && (s->flags & PIF_DROP_UNMATCHED))
-    {
-      (*path_options->matched) = TRUE;
-    }
 }
 
 static inline LogPipe *
@@ -383,11 +506,12 @@ log_pipe_clone(LogPipe *self)
   return self->clone(self);
 }
 
-static inline void
+static inline gint
 log_pipe_notify(LogPipe *s, gint notify_code, gpointer user_data)
 {
   if (s->notify)
-    s->notify(s, notify_code, user_data);
+    return s->notify(s, notify_code, user_data);
+  return NR_OK;
 }
 
 static inline void
@@ -402,6 +526,10 @@ log_pipe_set_persist_name(LogPipe *self, const gchar *persist_name);
 const gchar *
 log_pipe_get_persist_name(const LogPipe *self);
 
+void log_pipe_set_options(LogPipe *self, const LogPipeOptions *options);
+void log_pipe_set_internal(LogPipe *self, gboolean internal);
+gboolean log_pipe_is_internal(const LogPipe *self);
+
 void log_pipe_free_method(LogPipe *s);
 
 static inline GList *
