@@ -22,12 +22,13 @@
  *
  */
 
-#include "mock-transport.h"
-#include "proto_lib.h"
-#include "msg_parse_lib.h"
+#include <criterion/criterion.h>
+#include "libtest/mock-transport.h"
+#include "libtest/proto_lib.h"
+#include "libtest/msg_parse_lib.h"
+
 #include "logproto/logproto-record-server.h"
 
-#include <criterion/criterion.h>
 
 Test(log_proto, test_log_proto_binary_record_server_no_encoding)
 {
