@@ -21,11 +21,12 @@
  *
  */
 #include <criterion/criterion.h>
-#include <stdio.h>
-#include "stopwatch.h"
+#include "libtest/stopwatch.h"
+
 #include "scratch-buffers.h"
 #include "apphook.h"
 #include "kv-scanner.h"
+#include <stdio.h>
 
 static gboolean
 _expect_no_more_tokens(KVScanner *scanner, gchar **error)
@@ -139,14 +140,21 @@ typedef struct Testcase_t
   KVContainer expected;
 } Testcase;
 
-KVScanner *
-create_kv_scanner(const ScannerConfig config)
+static KVScanner *
+create_kv_scanner(const ScannerConfig *config)
 {
-  KVScanner *scanner = kv_scanner_new(config.kv_separator, config.pair_separator, config.extract_stray_words);
-  scanner->transform_value = config.transform_value;
+  KVScanner *scanner = g_new(KVScanner, 1);
+  kv_scanner_init(scanner, config->kv_separator, config->pair_separator, config->extract_stray_words);
+  kv_scanner_set_transform_value(scanner, config->transform_value);
   return scanner;
 }
 
+static inline void
+kv_scanner_free(KVScanner *scanner)
+{
+  g_free(scanner);
+}
+
 #define VARARG_STRUCT(VARARG_STRUCT_cont, VARARG_STRUCT_elem, ...) \
   (const VARARG_STRUCT_cont) { \
     .n = sizeof((const VARARG_STRUCT_elem[]) { __VA_ARGS__ }) / sizeof(VARARG_STRUCT_elem), \
@@ -189,7 +197,7 @@ _expect_kvq_triplets(KVScanner *scanner, KVQContainer args, gchar **error)
 
 #define _IMPL_EXPECT_KVQ(SCANNER_config, TEST_KV_SCAN_input, ...) \
   do { \
-    KVScanner *scanner = create_kv_scanner(SCANNER_config); \
+    KVScanner *scanner = create_kv_scanner(&SCANNER_config); \
     gchar *error = NULL; \
     \
     kv_scanner_input(scanner, TEST_KV_SCAN_input);            \
@@ -210,7 +218,7 @@ _expect_kvq_triplets(KVScanner *scanner, KVQContainer args, gchar **error)
 
 #define _IMPL_EXPECT_KV(TEST_KV_SCAN_config, TEST_KV_SCAN_input, ...) \
   do { \
-    KVScanner *scanner = create_kv_scanner(TEST_KV_SCAN_config); \
+    KVScanner *scanner = create_kv_scanner(&TEST_KV_SCAN_config); \
     gchar *error = NULL; \
     \
     kv_scanner_input(scanner, TEST_KV_SCAN_input);            \
@@ -236,7 +244,7 @@ _expect_kvq_triplets(KVScanner *scanner, KVQContainer args, gchar **error)
 
 #define _EXPECT_KV_AND_STRAY_WORDS(INPUT, STRAY, ...) \
   do { \
-    KVScanner *scanner = create_kv_scanner(((ScannerConfig) {     \
+    KVScanner *scanner = create_kv_scanner(&((ScannerConfig) {     \
       .kv_separator = '=',              \
       .extract_stray_words=TRUE})); \
     gchar *error = NULL; \
@@ -817,7 +825,7 @@ Try to increase stolen memory size if available in BIOS.",
     {}
   };
 
-  return g_memdup(tc, sizeof(tc));
+  return g_memdup2(tc, sizeof(tc));
 }
 
 static Testcase *
@@ -956,7 +964,7 @@ _provide_cases_for_performance_test_parse_long_msg(void)
     },
     {}
   };
-  return g_memdup(tc, sizeof(tc));
+  return g_memdup2(tc, sizeof(tc));
 }
 
 #define ITERATION_NUMBER 100000
