@@ -36,28 +36,11 @@
 
 #include <string.h>
 
-static void
-_reset_counter(StatsCluster *sc, gint type, StatsCounterItem *counter, gpointer user_data)
-{
-  stats_counter_set(counter, 0);
-}
 
 static inline void
 _reset_counter_if_needed(StatsCluster *sc, gint type, StatsCounterItem *counter, gpointer user_data)
 {
-  if (stats_counter_read_only(counter))
-    return;
-
-  if (strcmp(stats_cluster_get_type_name(sc, type), "memory_usage") == 0)
-    return;
-
-  switch (type)
-    {
-    case SC_TYPE_QUEUED:
-      return;
-    default:
-      _reset_counter(sc, type, counter, user_data);
-    }
+  stats_cluster_reset_counter_if_needed(sc, counter);
 }
 
 static void
