@@ -24,7 +24,7 @@
 #include "messages.h"
 #include "stats-query-commands.h"
 #include "stats/stats-query.h"
-#include "control/control-server.h"
+#include "control/control-connection.h"
 
 typedef enum _QueryCommand
 {
@@ -54,17 +54,21 @@ _append_reset_msg_if_found_matching_counters(gboolean found_match, GString *resu
 }
 
 static gboolean
-_ctl_format_get(StatsCounterItem *ctr, gpointer user_data)
+_ctl_format_get(gpointer user_data)
 {
-  GString *str = (GString *)user_data;
+  gpointer *args = (gpointer *) user_data;
+  StatsCounterItem *ctr = (StatsCounterItem *) args[0];
+  GString *str = (GString *)args[1];
   g_string_append_printf(str, "%s=%"G_GSIZE_FORMAT"\n", stats_counter_get_name(ctr), stats_counter_get(ctr));
   return TRUE;
 }
 
 static gboolean
-_ctl_format_name_without_value(StatsCounterItem *ctr, gpointer user_data)
+_ctl_format_name_without_value(gpointer user_data)
 {
-  GString *str = (GString *)user_data;
+  gpointer *args = (gpointer *) user_data;
+  StatsCounterItem *ctr = (StatsCounterItem *) args[0];
+  GString *str = (GString *)args[1];
   g_string_append_printf(str, "%s\n", stats_counter_get_name(ctr));
   return TRUE;
 }
@@ -182,8 +186,8 @@ _dispatch_query(gint cmd_id, const gchar *filter_expr, GString *result)
   return QUERY_CMDS[cmd_id](filter_expr, result);
 }
 
-static GString *
-_process_query_command(ControlConnection *cc, GString *command, gpointer user_data)
+void
+process_query_command(ControlConnection *cc, GString *command, gpointer user_data, gboolean *cancelled)
 {
   GString *result = g_string_new("");
   gchar **cmds = g_strsplit(command->str, " ", 3);
@@ -197,11 +201,5 @@ _process_query_command(ControlConnection *cc, GString *command, gpointer user_da
   if (result->len == 0)
     g_string_assign(result, "\n");
 
-  return result;
-}
-
-void
-process_query_command(ControlConnection *cc, GString *command, gpointer user_data)
-{
-  control_connection_start_as_thread(cc, _process_query_command, command, user_data);
+  control_connection_send_reply(cc, result);
 }
