@@ -26,8 +26,9 @@
  * connectivity.
  */
 
+#include <criterion/criterion.h>
+
 #include "host-resolve.h"
-#include "testutils.h"
 #include "apphook.h"
 #include "dnscache.h"
 #include "gsocket.h"
@@ -35,29 +36,6 @@
 #include "cfg.h"
 #include <libgen.h>
 
-#define HOST_RESOLVE_TESTCASE(x, ...) do { host_resolve_testcase_begin(domain_override, #x, #__VA_ARGS__); x(__VA_ARGS__); host_resolve_testcase_end(); } while(0)
-
-
-#define host_resolve_testcase_begin(domain_override, func, args)    \
-  do                                                              \
-    {                                                             \
-      testcase_begin("%s(%s)", func, args);                       \
-      dns_caching_thread_init();          \
-      host_resolve_options_defaults(&host_resolve_options);   \
-      host_resolve_options_init(&host_resolve_options, configuration);  \
-      hostname_reinit(NULL);            \
-    }                                                             \
-  while (0)
-
-#define host_resolve_testcase_end()                             \
-  do                                                            \
-    {                                                           \
-      host_resolve_options_destroy(&host_resolve_options);  \
-      dns_caching_thread_deinit();        \
-      testcase_end();                                           \
-    }                                                           \
-  while (0)
-
 static HostResolveOptions host_resolve_options;
 
 static void
@@ -69,8 +47,8 @@ assert_sockaddr_to_hostname(GSockAddr *sa, const gchar *expected)
   result = resolve_sockaddr_to_hostname(&result_len, sa, &host_resolve_options);
   g_sockaddr_unref(sa);
 
-  assert_string(result, expected, "resolved name mismatch");
-  assert_gint(result_len, strlen(result), "returned length is not true");
+  cr_assert_str_eq(result, expected, "resolved name mismatch");
+  cr_assert_eq(result_len, strlen(result), "returned length is not true");
 }
 
 static void
@@ -127,9 +105,9 @@ assert_hostname_to_sockaddr(gint family, const gchar *hostname, const gchar *exp
       g_sockaddr_unref(sa);
     }
 
-  assert_true(result, "unexpected error return");
-  assert_true(sa != NULL, "sockaddr can't be NULL for successful returns");
-  assert_string(ip, expected_ip, "resolved address mismatch");
+  cr_assert(result, "unexpected error return");
+  cr_assert_not_null(sa, "sockaddr can't be NULL for successful returns");
+  cr_assert_str_eq(ip, expected_ip, "resolved address mismatch");
 }
 
 static void
@@ -141,8 +119,8 @@ assert_hostname_to_sockaddr_fails(gint family, const gchar *hostname)
   result = resolve_hostname_to_sockaddr(&sa, family, hostname);
   g_sockaddr_unref(sa);
 
-  assert_null(sa, "returned sockaddr is non-NULL");
-  assert_false(result, "unexpected success returned");
+  cr_assert_null(sa, "returned sockaddr is non-NULL");
+  cr_assert_not(result, "unexpected success returned");
 }
 
 static void
@@ -152,8 +130,8 @@ assert_hostname_to_hostname_len(gsize buflen, const gchar *hostname, const gchar
   gsize result_len = 9999;
 
   result = resolve_hostname_to_hostname(&result_len, hostname, &host_resolve_options);
-  assert_string(result, expected, "hostname to hostname result mismatch");
-  assert_gint(result_len, strlen(result), "returned length is not true");
+  cr_assert_str_eq(result, expected, "hostname to hostname result mismatch");
+  cr_assert_eq(result_len, strlen(result), "returned length is not true");
 }
 
 static void
@@ -166,8 +144,7 @@ assert_hostname_to_hostname(const gchar *hostname, const gchar *expected)
   for (host_resolve_options.use_dns_cache = 0; host_resolve_options.use_dns_cache < 2; host_resolve_options.use_dns_cache++)  \
     for (host_resolve_options.normalize_hostnames = 0; host_resolve_options.normalize_hostnames < 2; host_resolve_options.normalize_hostnames++)
 
-static void
-test_resolvable_ip_results_in_hostname(void)
+Test(resolve_hostname, test_resolvable_ip_results_in_hostname)
 {
   host_resolve_options.use_dns = TRUE;
   for_all_resolve_cases()
@@ -180,8 +157,7 @@ test_resolvable_ip_results_in_hostname(void)
   }
 }
 
-static void
-test_unresolvable_ip_results_in_ip(void)
+Test(resolve_hostname, test_unresolvable_ip_results_in_ip)
 {
   fprintf(stderr, "The testcase %s takes a lot of time, it is advisable to turn it\n"
           "off for short iterations and reenable it at the end of the session.\n"
@@ -208,8 +184,7 @@ test_unresolvable_ip_results_in_ip(void)
   }
 }
 
-static void
-test_sockaddr_without_dns_resolution_results_in_ip(void)
+Test(resolve_hostname, test_sockaddr_without_dns_resolution_results_in_ip)
 {
   host_resolve_options.use_dns = FALSE;
   for_all_resolve_cases()
@@ -222,8 +197,7 @@ test_sockaddr_without_dns_resolution_results_in_ip(void)
   }
 }
 
-static void
-test_unix_domain_sockaddr_results_in_the_local_hostname(void)
+Test(resolve_hostname, test_unix_domain_sockaddr_results_in_the_local_hostname)
 {
   const gchar *local_host;
 
@@ -233,17 +207,7 @@ test_unix_domain_sockaddr_results_in_the_local_hostname(void)
   assert_sockaddr_to_hostname(NULL, local_host);
 }
 
-static void
-test_resolve_sockaddr_to_hostname(void)
-{
-  HOST_RESOLVE_TESTCASE(test_resolvable_ip_results_in_hostname);
-  HOST_RESOLVE_TESTCASE(test_unresolvable_ip_results_in_ip);
-  HOST_RESOLVE_TESTCASE(test_sockaddr_without_dns_resolution_results_in_ip);
-  HOST_RESOLVE_TESTCASE(test_unix_domain_sockaddr_results_in_the_local_hostname);
-}
-
-static void
-test_resolvable_hostname_results_in_sockaddr(void)
+Test(resolve_hostname, test_resolvable_hostname_results_in_sockaddr)
 {
   assert_hostname_to_sockaddr(AF_INET, "a.root-servers.net", "198.41.0.4");
   assert_hostname_to_sockaddr(AF_INET, "", "0.0.0.0");
@@ -253,21 +217,12 @@ test_resolvable_hostname_results_in_sockaddr(void)
 #endif
 }
 
-static void
-test_unresolvable_hostname_results_in_error(void)
+Test(resolve_hostname, test_unresolvable_hostname_results_in_error)
 {
   assert_hostname_to_sockaddr_fails(AF_INET, "foo.bar.baz");
 }
 
-static void
-test_resolve_hostname_to_sockaddr(void)
-{
-  HOST_RESOLVE_TESTCASE(test_resolvable_hostname_results_in_sockaddr);
-  HOST_RESOLVE_TESTCASE(test_unresolvable_hostname_results_in_error);
-}
-
-static void
-test_short_hostname_is_converted_to_fqdn_if_use_fqdn_is_set(void)
+Test(resolve_hostname, test_short_hostname_is_converted_to_fqdn_if_use_fqdn_is_set)
 {
   host_resolve_options.use_fqdn = TRUE;
 
@@ -278,16 +233,14 @@ test_short_hostname_is_converted_to_fqdn_if_use_fqdn_is_set(void)
   assert_hostname_to_hostname("foo", "foo.bardomain");
 }
 
-static void
-test_fqdn_hostname_is_converted_to_short_if_use_fqdn_is_unset(void)
+Test(resolve_hostname, test_fqdn_hostname_is_converted_to_short_if_use_fqdn_is_unset)
 {
   host_resolve_options.use_fqdn = FALSE;
 
   assert_hostname_to_hostname("foo.bardomain", "foo");
 }
 
-static void
-test_hostname_is_normalized_if_normalize_hostnames_is_set(void)
+Test(resolve_hostname, test_hostname_is_normalized_if_normalize_hostnames_is_set)
 {
   host_resolve_options.use_fqdn = TRUE;
   host_resolve_options.normalize_hostnames = TRUE;
@@ -296,22 +249,21 @@ test_hostname_is_normalized_if_normalize_hostnames_is_set(void)
 }
 
 static void
-test_resolve_hostname_to_hostname(void)
-{
-  HOST_RESOLVE_TESTCASE(test_short_hostname_is_converted_to_fqdn_if_use_fqdn_is_set);
-  HOST_RESOLVE_TESTCASE(test_fqdn_hostname_is_converted_to_short_if_use_fqdn_is_unset);
-  HOST_RESOLVE_TESTCASE(test_hostname_is_normalized_if_normalize_hostnames_is_set);
-}
-
-int
-main(int argc, char *argv[])
+setup(void)
 {
   app_startup();
-
-  configuration = cfg_new(VERSION_VALUE);
-  test_resolve_hostname_to_hostname();
-  test_resolve_hostname_to_sockaddr();
-  test_resolve_sockaddr_to_hostname();
-  cfg_free(configuration);
-  return 0;
+  configuration = cfg_new_snippet();
+  host_resolve_options_defaults(&host_resolve_options);
+  host_resolve_options_init(&host_resolve_options, &configuration->host_resolve_options);
+  hostname_reinit(NULL);
 }
+
+static void
+teardown(void)
+{
+  host_resolve_options_destroy(&host_resolve_options);
+  cfg_free(configuration);
+  app_shutdown();
+}
+
+TestSuite(resolve_hostname, .init = setup, .fini = teardown);
