@@ -41,6 +41,7 @@ static CfgLexerKeyword add_contextual_data_keywords[] =
   {"default_selector", KW_DEFAULT_SELECTOR},
   {"prefix", KW_PREFIX},
   {"filters", KW_FILTERS},
+  {"glob", KW_GLOB},
   {"ignore_case", KW_IGNORE_CASE},
   {NULL}
 };
@@ -57,4 +58,4 @@ CfgParser add_contextual_data_parser =
   .cleanup = (void (*)(gpointer)) log_pipe_unref,
 };
 
-CFG_PARSER_IMPLEMENT_LEXER_BINDING(add_contextual_data_, LogParser **)
+CFG_PARSER_IMPLEMENT_LEXER_BINDING(add_contextual_data_, ADD_CONTEXTUAL_DATA_, LogParser **)
