@@ -66,4 +66,4 @@ CfgParser afamqp_parser =
   .cleanup = (void (*)(gpointer)) log_pipe_unref,
 };
 
-CFG_PARSER_IMPLEMENT_LEXER_BINDING(afamqp_, LogDriver **)
+CFG_PARSER_IMPLEMENT_LEXER_BINDING(afamqp_, AFAMQP_, LogDriver **)
