@@ -73,39 +73,42 @@ extern int affile_debug;
     LL_CONTEXT_BLOCK_REF = 9,      /* LL_CONTEXT_BLOCK_REF  */
     LL_CONTEXT_BLOCK_CONTENT = 10, /* LL_CONTEXT_BLOCK_CONTENT  */
     LL_CONTEXT_BLOCK_ARG = 11,     /* LL_CONTEXT_BLOCK_ARG  */
-    LL_CONTEXT_PRAGMA = 12,        /* LL_CONTEXT_PRAGMA  */
-    LL_CONTEXT_FORMAT = 13,        /* LL_CONTEXT_FORMAT  */
-    LL_CONTEXT_TEMPLATE_FUNC = 14, /* LL_CONTEXT_TEMPLATE_FUNC  */
-    LL_CONTEXT_INNER_DEST = 15,    /* LL_CONTEXT_INNER_DEST  */
-    LL_CONTEXT_INNER_SRC = 16,     /* LL_CONTEXT_INNER_SRC  */
-    LL_CONTEXT_CLIENT_PROTO = 17,  /* LL_CONTEXT_CLIENT_PROTO  */
-    LL_CONTEXT_SERVER_PROTO = 18,  /* LL_CONTEXT_SERVER_PROTO  */
-    LL_CONTEXT_OPTIONS = 19,       /* LL_CONTEXT_OPTIONS  */
-    LL_CONTEXT_CONFIG = 20,        /* LL_CONTEXT_CONFIG  */
-    LL_CONTEXT_TEMPLATE_REF = 21,  /* LL_CONTEXT_TEMPLATE_REF  */
-    LL_CONTEXT_FILTERX = 22,       /* LL_CONTEXT_FILTERX  */
-    LL_CONTEXT_FILTERX_FUNC = 23,  /* LL_CONTEXT_FILTERX_FUNC  */
-    LL_CONTEXT_FILTERX_ENUM = 24,  /* LL_CONTEXT_FILTERX_ENUM  */
-    LL_CONTEXT_MAX = 25,           /* LL_CONTEXT_MAX  */
+    LL_CONTEXT_BLOCK_FUNCARG = 12, /* LL_CONTEXT_BLOCK_FUNCARG  */
+    LL_CONTEXT_PRAGMA = 13,        /* LL_CONTEXT_PRAGMA  */
+    LL_CONTEXT_FORMAT = 14,        /* LL_CONTEXT_FORMAT  */
+    LL_CONTEXT_TEMPLATE_FUNC = 15, /* LL_CONTEXT_TEMPLATE_FUNC  */
+    LL_CONTEXT_INNER_DEST = 16,    /* LL_CONTEXT_INNER_DEST  */
+    LL_CONTEXT_INNER_SRC = 17,     /* LL_CONTEXT_INNER_SRC  */
+    LL_CONTEXT_CLIENT_PROTO = 18,  /* LL_CONTEXT_CLIENT_PROTO  */
+    LL_CONTEXT_SERVER_PROTO = 19,  /* LL_CONTEXT_SERVER_PROTO  */
+    LL_CONTEXT_OPTIONS = 20,       /* LL_CONTEXT_OPTIONS  */
+    LL_CONTEXT_CONFIG = 21,        /* LL_CONTEXT_CONFIG  */
+    LL_CONTEXT_TEMPLATE_REF = 22,  /* LL_CONTEXT_TEMPLATE_REF  */
+    LL_CONTEXT_FILTERX = 23,       /* LL_CONTEXT_FILTERX  */
+    LL_CONTEXT_FILTERX_SIMPLE_FUNC = 24, /* LL_CONTEXT_FILTERX_SIMPLE_FUNC  */
+    LL_CONTEXT_FILTERX_ENUM = 25,  /* LL_CONTEXT_FILTERX_ENUM  */
+    LL_CONTEXT_FILTERX_FUNC = 26,  /* LL_CONTEXT_FILTERX_FUNC  */
+    LL_CONTEXT_MAX = 27,           /* LL_CONTEXT_MAX  */
     KW_ASSIGN = 9000,              /* KW_ASSIGN  */
-    KW_OR = 9001,                  /* KW_OR  */
-    KW_AND = 9002,                 /* KW_AND  */
-    KW_STR_EQ = 9003,              /* KW_STR_EQ  */
-    KW_STR_NE = 9004,              /* KW_STR_NE  */
-    KW_TA_EQ = 9005,               /* KW_TA_EQ  */
-    KW_TA_NE = 9006,               /* KW_TA_NE  */
-    KW_TAV_EQ = 9007,              /* KW_TAV_EQ  */
-    KW_TAV_NE = 9008,              /* KW_TAV_NE  */
-    KW_STR_LT = 9009,              /* KW_STR_LT  */
-    KW_STR_LE = 9010,              /* KW_STR_LE  */
-    KW_STR_GE = 9011,              /* KW_STR_GE  */
-    KW_STR_GT = 9012,              /* KW_STR_GT  */
-    KW_TA_LT = 9013,               /* KW_TA_LT  */
-    KW_TA_LE = 9014,               /* KW_TA_LE  */
-    KW_TA_GE = 9015,               /* KW_TA_GE  */
-    KW_TA_GT = 9016,               /* KW_TA_GT  */
-    KW_PLUS_ASSIGN = 9018,         /* KW_PLUS_ASSIGN  */
-    KW_NOT = 9017,                 /* KW_NOT  */
+    KW_OR = 9010,                  /* KW_OR  */
+    KW_AND = 9020,                 /* KW_AND  */
+    KW_STR_EQ = 9030,              /* KW_STR_EQ  */
+    KW_STR_NE = 9031,              /* KW_STR_NE  */
+    KW_TA_EQ = 9032,               /* KW_TA_EQ  */
+    KW_TA_NE = 9033,               /* KW_TA_NE  */
+    KW_TAV_EQ = 9034,              /* KW_TAV_EQ  */
+    KW_TAV_NE = 9035,              /* KW_TAV_NE  */
+    KW_STR_LT = 9040,              /* KW_STR_LT  */
+    KW_STR_LE = 9041,              /* KW_STR_LE  */
+    KW_STR_GE = 9042,              /* KW_STR_GE  */
+    KW_STR_GT = 9043,              /* KW_STR_GT  */
+    KW_TA_LT = 9044,               /* KW_TA_LT  */
+    KW_TA_LE = 9045,               /* KW_TA_LE  */
+    KW_TA_GE = 9046,               /* KW_TA_GE  */
+    KW_TA_GT = 9047,               /* KW_TA_GT  */
+    KW_PLUS_ASSIGN = 9050,         /* KW_PLUS_ASSIGN  */
+    KW_REGEXP_MATCH = 9060,        /* KW_REGEXP_MATCH  */
+    KW_NOT = 9049,                 /* KW_NOT  */
     KW_SOURCE = 10000,             /* KW_SOURCE  */
     KW_FILTER = 10001,             /* KW_FILTER  */
     KW_PARSER = 10002,             /* KW_PARSER  */
@@ -269,18 +272,19 @@ extern int affile_debug;
     KW_PIPE = 10533,               /* KW_PIPE  */
     KW_FSYNC = 10534,              /* KW_FSYNC  */
     KW_FOLLOW_FREQ = 10535,        /* KW_FOLLOW_FREQ  */
-    KW_OVERWRITE_IF_OLDER = 10536, /* KW_OVERWRITE_IF_OLDER  */
-    KW_SYMLINK_AS = 10537,         /* KW_SYMLINK_AS  */
-    KW_MULTI_LINE_TIMEOUT = 10538, /* KW_MULTI_LINE_TIMEOUT  */
-    KW_WILDCARD_FILE = 10539,      /* KW_WILDCARD_FILE  */
-    KW_BASE_DIR = 10540,           /* KW_BASE_DIR  */
-    KW_FILENAME_PATTERN = 10541,   /* KW_FILENAME_PATTERN  */
-    KW_RECURSIVE = 10542,          /* KW_RECURSIVE  */
-    KW_MAX_FILES = 10543,          /* KW_MAX_FILES  */
-    KW_MONITOR_METHOD = 10544,     /* KW_MONITOR_METHOD  */
-    KW_FORCE_DIRECTORY_POLLING = 10545, /* KW_FORCE_DIRECTORY_POLLING  */
-    KW_STDIN = 10546,              /* KW_STDIN  */
-    KW_STDOUT = 10547              /* KW_STDOUT  */
+    KW_MONITOR_FREQ = 10536,       /* KW_MONITOR_FREQ  */
+    KW_OVERWRITE_IF_OLDER = 10537, /* KW_OVERWRITE_IF_OLDER  */
+    KW_SYMLINK_AS = 10538,         /* KW_SYMLINK_AS  */
+    KW_MULTI_LINE_TIMEOUT = 10539, /* KW_MULTI_LINE_TIMEOUT  */
+    KW_WILDCARD_FILE = 10540,      /* KW_WILDCARD_FILE  */
+    KW_BASE_DIR = 10541,           /* KW_BASE_DIR  */
+    KW_FILENAME_PATTERN = 10542,   /* KW_FILENAME_PATTERN  */
+    KW_RECURSIVE = 10543,          /* KW_RECURSIVE  */
+    KW_MAX_FILES = 10544,          /* KW_MAX_FILES  */
+    KW_MONITOR_METHOD = 10545,     /* KW_MONITOR_METHOD  */
+    KW_FORCE_DIRECTORY_POLLING = 10546, /* KW_FORCE_DIRECTORY_POLLING  */
+    KW_STDIN = 10547,              /* KW_STDIN  */
+    KW_STDOUT = 10548              /* KW_STDOUT  */
   };
   typedef enum affile_tokentype affile_token_kind_t;
 #endif
@@ -300,39 +304,42 @@ extern int affile_debug;
 #define LL_CONTEXT_BLOCK_REF 9
 #define LL_CONTEXT_BLOCK_CONTENT 10
 #define LL_CONTEXT_BLOCK_ARG 11
-#define LL_CONTEXT_PRAGMA 12
-#define LL_CONTEXT_FORMAT 13
-#define LL_CONTEXT_TEMPLATE_FUNC 14
-#define LL_CONTEXT_INNER_DEST 15
-#define LL_CONTEXT_INNER_SRC 16
-#define LL_CONTEXT_CLIENT_PROTO 17
-#define LL_CONTEXT_SERVER_PROTO 18
-#define LL_CONTEXT_OPTIONS 19
-#define LL_CONTEXT_CONFIG 20
-#define LL_CONTEXT_TEMPLATE_REF 21
-#define LL_CONTEXT_FILTERX 22
-#define LL_CONTEXT_FILTERX_FUNC 23
-#define LL_CONTEXT_FILTERX_ENUM 24
-#define LL_CONTEXT_MAX 25
+#define LL_CONTEXT_BLOCK_FUNCARG 12
+#define LL_CONTEXT_PRAGMA 13
+#define LL_CONTEXT_FORMAT 14
+#define LL_CONTEXT_TEMPLATE_FUNC 15
+#define LL_CONTEXT_INNER_DEST 16
+#define LL_CONTEXT_INNER_SRC 17
+#define LL_CONTEXT_CLIENT_PROTO 18
+#define LL_CONTEXT_SERVER_PROTO 19
+#define LL_CONTEXT_OPTIONS 20
+#define LL_CONTEXT_CONFIG 21
+#define LL_CONTEXT_TEMPLATE_REF 22
+#define LL_CONTEXT_FILTERX 23
+#define LL_CONTEXT_FILTERX_SIMPLE_FUNC 24
+#define LL_CONTEXT_FILTERX_ENUM 25
+#define LL_CONTEXT_FILTERX_FUNC 26
+#define LL_CONTEXT_MAX 27
 #define KW_ASSIGN 9000
-#define KW_OR 9001
-#define KW_AND 9002
-#define KW_STR_EQ 9003
-#define KW_STR_NE 9004
-#define KW_TA_EQ 9005
-#define KW_TA_NE 9006
-#define KW_TAV_EQ 9007
-#define KW_TAV_NE 9008
-#define KW_STR_LT 9009
-#define KW_STR_LE 9010
-#define KW_STR_GE 9011
-#define KW_STR_GT 9012
-#define KW_TA_LT 9013
-#define KW_TA_LE 9014
-#define KW_TA_GE 9015
-#define KW_TA_GT 9016
-#define KW_PLUS_ASSIGN 9018
-#define KW_NOT 9017
+#define KW_OR 9010
+#define KW_AND 9020
+#define KW_STR_EQ 9030
+#define KW_STR_NE 9031
+#define KW_TA_EQ 9032
+#define KW_TA_NE 9033
+#define KW_TAV_EQ 9034
+#define KW_TAV_NE 9035
+#define KW_STR_LT 9040
+#define KW_STR_LE 9041
+#define KW_STR_GE 9042
+#define KW_STR_GT 9043
+#define KW_TA_LT 9044
+#define KW_TA_LE 9045
+#define KW_TA_GE 9046
+#define KW_TA_GT 9047
+#define KW_PLUS_ASSIGN 9050
+#define KW_REGEXP_MATCH 9060
+#define KW_NOT 9049
 #define KW_SOURCE 10000
 #define KW_FILTER 10001
 #define KW_PARSER 10002
@@ -496,18 +503,19 @@ extern int affile_debug;
 #define KW_PIPE 10533
 #define KW_FSYNC 10534
 #define KW_FOLLOW_FREQ 10535
-#define KW_OVERWRITE_IF_OLDER 10536
-#define KW_SYMLINK_AS 10537
-#define KW_MULTI_LINE_TIMEOUT 10538
-#define KW_WILDCARD_FILE 10539
-#define KW_BASE_DIR 10540
-#define KW_FILENAME_PATTERN 10541
-#define KW_RECURSIVE 10542
-#define KW_MAX_FILES 10543
-#define KW_MONITOR_METHOD 10544
-#define KW_FORCE_DIRECTORY_POLLING 10545
-#define KW_STDIN 10546
-#define KW_STDOUT 10547
+#define KW_MONITOR_FREQ 10536
+#define KW_OVERWRITE_IF_OLDER 10537
+#define KW_SYMLINK_AS 10538
+#define KW_MULTI_LINE_TIMEOUT 10539
+#define KW_WILDCARD_FILE 10540
+#define KW_BASE_DIR 10541
+#define KW_FILENAME_PATTERN 10542
+#define KW_RECURSIVE 10543
+#define KW_MAX_FILES 10544
+#define KW_MONITOR_METHOD 10545
+#define KW_FORCE_DIRECTORY_POLLING 10546
+#define KW_STDIN 10547
+#define KW_STDOUT 10548
 
 /* Value type.  */
 #if ! defined AFFILE_STYPE && ! defined AFFILE_STYPE_IS_DECLARED
