@@ -35,6 +35,7 @@ static CfgLexerKeyword affile_keywords[] =
   { "fifo",               KW_PIPE },
   { "pipe",               KW_PIPE },
   { "stdin",              KW_STDIN },
+  { "stdout",             KW_STDOUT },
 
   { "wildcard_file",      KW_WILDCARD_FILE },
   { "base_dir",           KW_BASE_DIR },
@@ -49,6 +50,7 @@ static CfgLexerKeyword affile_keywords[] =
   { "overwrite_if_older", KW_OVERWRITE_IF_OLDER },
   { "symlink_as",         KW_SYMLINK_AS },
   { "follow_freq",        KW_FOLLOW_FREQ },
+  { "monitor_freq",       KW_MONITOR_FREQ },
   { "multi_line_timeout", KW_MULTI_LINE_TIMEOUT },
   { "time_reap",          KW_TIME_REAP },
   { NULL }
