@@ -44,11 +44,76 @@
 #include <errno.h>
 #include <time.h>
 #include <stdlib.h>
+#include <glib/gprintf.h>
 
 #include <iv.h>
 
-static inline const gchar *
+static inline gint G_GNUC_PRINTF(2, 3)
+_format_g_va_string(gchar **strp, const gchar *format, ...)
+{
+  va_list args;
+
+  va_start(args, format);
+  gint result = g_vasprintf(strp, format, args);
+  va_end(args);
+
+  return result;
+}
+
+static const gchar *
 _format_persist_name(const LogPipe *s)
+{
+  const FileReader *self = (const FileReader *)s;
+  guint name_hash = g_str_hash(self->filename->str);
+  const gint max_persist_name_len = 1024;
+  gchar *persist_name = NULL;
+
+  /* same wildcard file sources even with persist-name defined can be the same, so
+     file name hash is added as well to ensure uniquiness always
+     also adding now the filename (or at least an end part of it) for debug aid purpose
+   */
+  gint used_len;
+  if (self->owner->super.super.persist_name)
+    used_len = _format_g_va_string(&persist_name, "%s.%s.%u.curpos", self->persist_name_prefix,
+                                   self->owner->super.super.persist_name, name_hash);
+  else
+    used_len = _format_g_va_string(&persist_name, "%s.%u.curpos", self->persist_name_prefix, name_hash);
+  g_assert(used_len > 0);
+
+  if (used_len < max_persist_name_len - 2 - 1) /* () and the terminating NUL */
+    {
+      gchar *base_name = persist_name;
+      gint filename_len = strlen(self->filename->str);
+      gint remaining_len = max_persist_name_len - used_len - 2 - 1;  /* () and the terminating NUL */
+      persist_name = NULL; /* glib requires the input ptr to be NULL otherwise returning with -1 from g_vasprintf */
+      _format_g_va_string(&persist_name, "%s(%s)", base_name,
+                          self->filename->str + MAX(0, filename_len - remaining_len));
+      g_free(base_name);
+    }
+
+  return persist_name;
+}
+
+static inline const gchar *
+_generate_persist_name(const LogPipe *s)
+{
+  FileReader *self = (FileReader *) s;
+
+  if (self->persist_name == NULL)
+    {
+      if (self->super.generate_persist_name == _generate_persist_name)
+        self->persist_name = _format_persist_name(s);
+      else
+        {
+          const gchar *name = log_pipe_get_persist_name(s);
+          self->persist_name = name ? g_strdup(name) : _format_persist_name(s);
+        }
+    }
+  return self->persist_name;
+}
+
+static inline const gchar *
+_format_legacy_persist_name(const LogPipe *s)
 {
   const FileReader *self = (const FileReader *)s;
   static gchar persist_name[1024];
@@ -61,6 +126,26 @@ _format_persist_name(const LogPipe *s)
   return persist_name;
 }
 
+static gboolean
+_update_legacy_persist_name(FileReader *self)
+{
+  GlobalConfig *cfg = log_pipe_get_config(&self->super);
+
+  if (cfg->state == NULL)
+    return TRUE;
+
+  const gchar *current_persist_name = _generate_persist_name(&self->super);
+  const gchar *legacy_persist_name = _format_legacy_persist_name(&self->super);
+
+  if (persist_state_entry_exists(cfg->state, current_persist_name))
+    return TRUE;
+
+  if (FALSE == persist_state_entry_exists(cfg->state, legacy_persist_name))
+    return TRUE;
+
+  return persist_state_copy_entry(cfg->state, legacy_persist_name, current_persist_name);
+}
+
 static void
 _recover_state(LogPipe *s, GlobalConfig *cfg, LogProtoServer *proto)
 {
@@ -69,7 +154,7 @@ _recover_state(LogPipe *s, GlobalConfig *cfg, LogProtoServer *proto)
   if (!self->options->restore_state)
     return;
 
-  if (!log_proto_server_restart_with_state(proto, cfg->state, _format_persist_name(s)))
+  if (!log_proto_server_restart_with_state(proto, cfg->state, log_pipe_get_persist_name(s)))
     {
       msg_error("Error converting persistent state from on-disk format, losing file position information",
                 evt_tag_str("filename", self->filename->str));
@@ -77,6 +162,59 @@ _recover_state(LogPipe *s, GlobalConfig *cfg, LogProtoServer *proto)
     }
 }
 
+static gboolean
+_can_check_eof(gint fd)
+{
+  struct stat st;
+  return fstat(fd, &st) == 0 && S_ISFIFO(st.st_mode) == 0;
+}
+
+static gboolean
+_reader_check_eof(FileReader *self, gint fd)
+{
+  if (fd < 0)
+    return FALSE;
+
+  off_t pos = lseek(fd, 0, SEEK_CUR);
+  if (pos == (off_t) -1)
+    {
+      msg_error("Error invoking seek on file",
+                evt_tag_str("filename", self->filename->str),
+                evt_tag_error("error"));
+      return FALSE;
+    }
+
+  struct stat st;
+  gboolean end_of_file = fstat(fd, &st) == 0 && pos == st.st_size;
+  return end_of_file;
+}
+
+static inline gboolean
+_reader_on_eof(FileReader *self)
+{
+  gboolean result = TRUE;
+  if (log_pipe_notify(&self->super, NC_FILE_EOF, self) & NR_STOP_ON_EOF)
+    result = FALSE;
+  return result;
+}
+
+static inline gboolean
+_reader_check_watches(PollEvents *poll_events, gpointer user_data)
+{
+  FileReader *self = (FileReader *) user_data;
+  gboolean check_again = TRUE;
+  gint fd = poll_events_get_fd(poll_events);
+
+  if (_reader_check_eof(self, fd))
+    {
+      msg_trace("End of file, following file",
+                evt_tag_str("follow_filename", self->filename->str),
+                evt_tag_int("fn", fd));
+      check_again = _reader_on_eof(self);
+    }
+  return check_again;
+}
+
 static gboolean
 _is_fd_pollable(gint fd)
 {
@@ -96,27 +234,39 @@ _is_fd_pollable(gint fd)
 static PollEvents *
 _construct_poll_events(FileReader *self, gint fd)
 {
+  PollEvents *poll_events = NULL;
   if (self->options->follow_freq > 0)
     {
       LogProtoFileReaderOptions *proto_opts = file_reader_options_get_log_proto_options(self->options);
 
       if (proto_opts->multi_line_options.mode == MLM_NONE)
-        return poll_file_changes_new(fd, self->filename->str, self->options->follow_freq, &self->super);
+        poll_events = poll_file_changes_new(fd, self->filename->str, self->options->follow_freq, &self->super);
       else
-        return poll_multiline_file_changes_new(fd, self->filename->str, self->options->follow_freq,
-                                               self->options->multi_line_timeout, self);
+        poll_events = poll_multiline_file_changes_new(fd, self->filename->str, self->options->follow_freq,
+                                                      self->options->multi_line_timeout, self);
+      msg_trace("File follow-mode is syslog-ng poll");
     }
   else if (fd >= 0 && _is_fd_pollable(fd))
-    return poll_fd_events_new(fd);
+    {
+      poll_events = poll_fd_events_new(fd);
+      msg_trace("File follow-mode is ivykis poll");
+      msg_trace("Selected ivykis poll method", evt_tag_str("file_poll_method", iv_poll_method_name()));
+    }
   else
     {
       msg_error("Unable to determine how to monitor this file, follow_freq() unset and it is not possible to poll it "
                 "with the current ivykis polling method. Set follow-freq() for regular files or change "
                 "IV_EXCLUDE_POLL_METHOD environment variable to override the automatically selected polling method",
+                evt_tag_str("file_poll_method", iv_poll_method_name()),
                 evt_tag_str("filename", self->filename->str),
                 evt_tag_int("fd", fd));
       return NULL;
     }
+
+  if (_can_check_eof(fd))
+    poll_events_set_checker(poll_events, _reader_check_watches, self);
+
+  return poll_events;
 }
 
 static LogTransport *
@@ -130,12 +280,9 @@ _construct_proto(FileReader *self, gint fd)
 {
   LogReaderOptions *reader_options = &self->options->reader_options;
   LogProtoFileReaderOptions *proto_options = file_reader_options_get_log_proto_options(self->options);
-  LogTransport *transport;
-  MsgFormatHandler *format_handler;
+  LogTransport *transport = _construct_transport(self, fd);
+  MsgFormatHandler *format_handler = reader_options->parse_options.format_handler;
 
-  transport = _construct_transport(self, fd);
-
-  format_handler = reader_options->parse_options.format_handler;
   if ((format_handler && format_handler->construct_proto))
     {
       log_proto_server_options_set_ack_tracker_factory(&proto_options->super,
@@ -258,8 +405,51 @@ _reopen_on_notify(LogPipe *s, gboolean recover_state)
   _reader_open_file(s, recover_state);
 }
 
+static void
+_on_file_close(FileReader *self)
+{
+  if (self->options->exit_on_eof)
+    cfg_shutdown(log_pipe_get_config(&self->super));
+}
+
+static void
+_on_file_moved(FileReader *self)
+{
+  msg_verbose("Follow-mode file source moved, tracking of the new file is started",
+              evt_tag_str("filename", self->filename->str));
+  if (self->on_file_moved)
+    self->on_file_moved(self);
+  else
+    _reopen_on_notify(&self->super, TRUE);
+}
+
+static void
+_on_file_deleted(FileReader *self)
+{
+  /* We want to handle the events like file deleted in the same workflow.
+   * Manually polled file readers like poll-file-changes can handle this case,
+   * read the remaining file content to the end, and signal the EOF, this is
+   * happening from poll_events_update_watches, but that one is not always triggered
+   * automatically for system file event notifications (like poll-fd-events), as there
+   * might be no more file events after the file is fully read.
+   * So, we have to trigger one more read.
+   * NOTE: Do not try to close the reader directly from here, as there might already be
+   *       an io-operation in progress!
+   */
+  if (poll_events_system_polled(self->reader->poll_events))
+    log_reader_trigger_one_check(self->reader);
+}
+
+static void
+_on_read_error(FileReader *self)
+{
+  msg_verbose("Error while following source file, reopening in the hope it would work",
+              evt_tag_str("filename", self->filename->str));
+  _reopen_on_notify(&self->super, FALSE);
+}
+
 /* NOTE: runs in the main thread */
-void
+gint
 file_reader_notify_method(LogPipe *s, gint notify_code, gpointer user_data)
 {
   FileReader *self = (FileReader *) s;
@@ -267,43 +457,41 @@ file_reader_notify_method(LogPipe *s, gint notify_code, gpointer user_data)
   switch (notify_code)
     {
     case NC_CLOSE:
-      if (self->options->exit_on_eof)
-        cfg_shutdown(log_pipe_get_config(s));
+      _on_file_close(self);
       break;
 
     case NC_FILE_MOVED:
-      msg_verbose("Follow-mode file source moved, tracking of the new file is started",
-                  evt_tag_str("filename", self->filename->str));
-      _reopen_on_notify(s, TRUE);
+      _on_file_moved(self);
       break;
 
     case NC_READ_ERROR:
-      msg_verbose("Error while following source file, reopening in the hope it would work",
-                  evt_tag_str("filename", self->filename->str));
-      _reopen_on_notify(s, FALSE);
+      _on_read_error(self);
+      break;
+
+    case NC_FILE_DELETED:
+      _on_file_deleted(self);
       break;
 
     default:
       break;
     }
+  return NR_OK;
 }
 
 void
 file_reader_queue_method(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
 {
   FileReader *self = (FileReader *)s;
-  static NVHandle filename_handle = 0;
 
-  if (!filename_handle)
-    filename_handle = log_msg_get_value_handle("FILE_NAME");
-
-  log_msg_set_value(msg, filename_handle, self->filename->str, self->filename->len);
+  log_msg_set_value(msg, LM_V_FILE_NAME, self->filename->str, self->filename->len);
   log_pipe_forward_msg(s, msg, path_options);
 }
 
 gboolean
 file_reader_init_method(LogPipe *s)
 {
+  _update_legacy_persist_name((FileReader *)s);
+
   return _reader_open_file(s, TRUE);
 }
 
@@ -323,13 +511,14 @@ file_reader_free_method(LogPipe *s)
 
   g_assert(!self->reader);
   g_string_free(self->filename, TRUE);
+  g_free((gpointer)self->persist_name);
 }
 
 void
 file_reader_remove_persist_state(FileReader *self)
 {
   GlobalConfig *cfg = log_pipe_get_config(&self->super);
-  const gchar *old_persist_name = _format_persist_name(&self->super);
+  const gchar *old_persist_name = log_pipe_get_persist_name(&self->super);
   gchar *new_persist_name = g_strdup_printf("%s_REMOVED", old_persist_name);
   /* This is required to clean the persist entry from file during restart */
   persist_state_remove_entry(cfg->state, old_persist_name);
@@ -355,7 +544,8 @@ file_reader_cue_buffer_flush(FileReader *self)
 void
 file_reader_init_instance (FileReader *self, const gchar *filename,
                            FileReaderOptions *options, FileOpener *opener,
-                           LogSrcDriver *owner, GlobalConfig *cfg)
+                           LogSrcDriver *owner, GlobalConfig *cfg,
+                           const gchar *persist_name_prefix)
 {
   log_pipe_init_instance (&self->super, cfg);
   self->super.init = file_reader_init_method;
@@ -363,7 +553,8 @@ file_reader_init_instance (FileReader *self, const gchar *filename,
   self->super.deinit = file_reader_deinit_method;
   self->super.notify = file_reader_notify_method;
   self->super.free_fn = file_reader_free_method;
-  self->super.generate_persist_name = _format_persist_name;
+  self->super.generate_persist_name = _generate_persist_name;
+  self->persist_name_prefix = persist_name_prefix;
   self->filename = g_string_new (filename);
   self->options = options;
   self->opener = opener;
@@ -377,7 +568,7 @@ file_reader_new(const gchar *filename, FileReaderOptions *options, FileOpener *o
 {
   FileReader *self = g_new0(FileReader, 1);
 
-  file_reader_init_instance (self, filename, options, opener, owner, cfg);
+  file_reader_init_instance (self, filename, options, opener, owner, cfg, "affile_sd");
   return self;
 }
 
