@@ -35,7 +35,8 @@ typedef struct _FileReaderOptions
   gboolean exit_on_eof;
 } FileReaderOptions;
 
-typedef struct _FileReader
+typedef struct _FileReader FileReader;
+struct _FileReader
 {
   LogPipe super;
   LogSrcDriver *owner;
@@ -43,7 +44,11 @@ typedef struct _FileReader
   FileReaderOptions *options;
   FileOpener *opener;
   LogReader *reader;
-} FileReader;
+  const gchar *persist_name;
+  const gchar *persist_name_prefix;
+
+  void (*on_file_moved)(FileReader *);
+};
 
 static inline LogProtoFileReaderOptions *
 file_reader_options_get_log_proto_options(FileReaderOptions *options)
@@ -55,14 +60,13 @@ FileReader *file_reader_new(const gchar *filename, FileReaderOptions *options, F
                             GlobalConfig *cfg);
 
 void file_reader_init_instance(FileReader *self, const gchar *filename, FileReaderOptions *options, FileOpener *opener,
-                               LogSrcDriver *owner,
-                               GlobalConfig *cfg);
+                               LogSrcDriver *owner, GlobalConfig *cfg, const gchar *persist_name_prefix);
 
 gboolean file_reader_init_method(LogPipe *s);
 gboolean file_reader_deinit_method(LogPipe *s);
 void file_reader_free_method(LogPipe *s);
 void file_reader_queue_method(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options);
-void file_reader_notify_method(LogPipe *s, gint notify_code, gpointer user_data);
+gint file_reader_notify_method(LogPipe *s, gint notify_code, gpointer user_data);
 
 void file_reader_remove_persist_state(FileReader *self);
 void file_reader_stop_follow_file(FileReader *self);
