@@ -93,9 +93,9 @@ _construct_multitransport_with_tls_factory(TransportMapperInet *self, gint fd)
 }
 
 static LogTransport *
-_construct_tls_transport(TransportMapperInet *self, gint fd)
+_construct_tls_or_multi_transport(TransportMapperInet *self, gboolean create_multi_transport, gint fd)
 {
-  if (self->super.create_multitransport)
+  if (create_multi_transport)
     return _construct_multitransport_with_tls_factory(self, fd);
 
   TLSSession *tls_session = tls_context_setup_session(self->tls_context);
@@ -112,7 +112,6 @@ static LogTransport *
 _construct_multitransport_with_plain_tcp_factory(TransportMapperInet *self, gint fd)
 {
   TransportFactory *default_factory = transport_factory_socket_new(self->super.sock_type);
-
   return multitransport_new(default_factory, fd);
 }
 
@@ -128,9 +127,9 @@ _construct_multitransport_with_plain_and_tls_factories(TransportMapperInet *self
 }
 
 static LogTransport *
-_construct_plain_tcp_transport(TransportMapperInet *self, gint fd)
+_construct_plain_tcp_or_multi_transport(TransportMapperInet *self, gboolean create_multi_transport, gint fd)
 {
-  if (self->super.create_multitransport)
+  if (create_multi_transport)
     return _construct_multitransport_with_plain_tcp_factory(self, fd);
 
   if (self->super.sock_type == SOCK_DGRAM)
@@ -144,17 +143,27 @@ transport_mapper_inet_construct_log_transport(TransportMapper *s, gint fd)
 {
   TransportMapperInet *self = (TransportMapperInet *) s;
 
+  gboolean proxy_should_switch_transport = FALSE;
+  LogTransport *transport = NULL;
+
   if (self->tls_context && _is_tls_required(self))
     {
-      return _construct_tls_transport(self, fd);
+      transport = _construct_tls_or_multi_transport(self, self->super.create_multitransport, fd);
     }
-
-  if (self->tls_context)
+  else if (self->tls_context)
     {
-      return _construct_multitransport_with_plain_and_tls_factories(self, fd);
+      proxy_should_switch_transport = TRUE;
+      transport = _construct_multitransport_with_plain_and_tls_factories(self, fd);
+    }
+  else
+    {
+      transport = _construct_plain_tcp_or_multi_transport(self, self->super.create_multitransport, fd);
     }
 
-  return _construct_plain_tcp_transport(self, fd);
+  if (self->proxied)
+    log_transport_socket_proxy_new(transport, proxy_should_switch_transport);
+
+  return transport;
 }
 
 static gboolean
@@ -295,7 +304,6 @@ transport_mapper_inet_init_instance(TransportMapperInet *self, const gchar *tran
   self->super.address_family = AF_INET;
 }
 
-
 TransportMapperInet *
 transport_mapper_inet_new_instance(const gchar *transport)
 {
@@ -305,11 +313,28 @@ transport_mapper_inet_new_instance(const gchar *transport)
   return self;
 }
 
+static gboolean
+transport_mapper_tcp_apply_transport(TransportMapper *s, GlobalConfig *cfg)
+{
+  TransportMapperInet *self = (TransportMapperInet *) s;
+
+  if (!transport_mapper_inet_apply_transport_method(s, cfg))
+    return FALSE;
+
+  if (self->tls_context)
+    self->super.transport_name = g_strdup("rfc3164+tls");
+  else
+    self->super.transport_name = g_strdup("rfc3164+tcp");
+
+  return TRUE;
+}
+
 TransportMapper *
 transport_mapper_tcp_new(void)
 {
   TransportMapperInet *self = transport_mapper_inet_new_instance("tcp");
 
+  self->super.apply_transport = transport_mapper_tcp_apply_transport;
   self->super.sock_type = SOCK_STREAM;
   self->super.sock_proto = IPPROTO_TCP;
   self->super.logproto = "text";
@@ -334,6 +359,7 @@ transport_mapper_udp_new(void)
 {
   TransportMapperInet *self = transport_mapper_inet_new_instance("udp");
 
+  self->super.transport_name = g_strdup("rfc3164+udp");
   self->super.sock_type = SOCK_DGRAM;
   self->super.sock_proto = IPPROTO_UDP;
   self->super.logproto = "dgram";
@@ -367,15 +393,25 @@ transport_mapper_network_apply_transport(TransportMapper *s, GlobalConfig *cfg)
   self->server_port = NETWORK_PORT;
   if (strcasecmp(transport, "udp") == 0)
     {
+      self->super.logproto = "dgram";
       self->super.sock_type = SOCK_DGRAM;
       self->super.sock_proto = IPPROTO_UDP;
-      self->super.logproto = "dgram";
+      self->super.transport_name = g_strdup("rfc3164+udp");
     }
   else if (strcasecmp(transport, "tcp") == 0)
     {
       self->super.logproto = "text";
       self->super.sock_type = SOCK_STREAM;
       self->super.sock_proto = IPPROTO_TCP;
+      self->super.transport_name = g_strdup("rfc3164+tcp");
+    }
+  else if (strcasecmp(transport, "proxied-tcp") == 0)
+    {
+      self->super.logproto = "text";
+      self->super.sock_type = SOCK_STREAM;
+      self->super.sock_proto = IPPROTO_TCP;
+      self->proxied = TRUE;
+      self->super.transport_name = g_strdup("rfc3164+proxied-tcp");
     }
   else if (strcasecmp(transport, "tls") == 0)
     {
@@ -383,13 +419,25 @@ transport_mapper_network_apply_transport(TransportMapper *s, GlobalConfig *cfg)
       self->super.sock_type = SOCK_STREAM;
       self->super.sock_proto = IPPROTO_TCP;
       self->require_tls = TRUE;
+      self->super.transport_name = g_strdup("rfc3164+tls");
     }
   else if (strcasecmp(transport, "proxied-tls") == 0)
     {
-      self->super.logproto = "proxied-tcp";
+      self->super.logproto = "text";
       self->super.sock_type = SOCK_STREAM;
       self->super.sock_proto = IPPROTO_TCP;
+      self->proxied = TRUE;
       self->require_tls = TRUE;
+      self->super.transport_name = g_strdup("rfc3164+proxied-tls");
+    }
+  else if (strcasecmp(transport, "proxied-tls-passthrough") == 0)
+    {
+      self->super.logproto = "text";
+      self->super.sock_type = SOCK_STREAM;
+      self->super.sock_proto = IPPROTO_TCP;
+      self->proxied = TRUE;
+      self->allow_tls = TRUE;
+      self->super.transport_name = g_strdup("rfc3164+proxied-tls-passthrough");
     }
   else
     {
@@ -399,6 +447,7 @@ transport_mapper_network_apply_transport(TransportMapper *s, GlobalConfig *cfg)
       /* FIXME: look up port/protocol from the logproto */
       self->server_port = TCP_PORT;
       self->allow_tls = TRUE;
+      self->super.transport_name = g_strdup_printf("rfc3164+%s", self->super.transport);
     }
 
   g_assert(self->server_port != 0);
@@ -441,9 +490,10 @@ transport_mapper_syslog_apply_transport(TransportMapper *s, GlobalConfig *cfg)
       else
         self->server_port = SYSLOG_TRANSPORT_UDP_PORT;
 
+      self->super.logproto = "dgram";
       self->super.sock_type = SOCK_DGRAM;
       self->super.sock_proto = IPPROTO_UDP;
-      self->super.logproto = "dgram";
+      self->super.transport_name = g_strdup("rfc5426");
     }
   else if (strcasecmp(transport, "tcp") == 0)
     {
@@ -451,6 +501,16 @@ transport_mapper_syslog_apply_transport(TransportMapper *s, GlobalConfig *cfg)
       self->super.logproto = "framed";
       self->super.sock_type = SOCK_STREAM;
       self->super.sock_proto = IPPROTO_TCP;
+      self->super.transport_name = g_strdup("rfc6587");
+    }
+  else if (strcasecmp(transport, "proxied-tcp") == 0)
+    {
+      self->server_port = SYSLOG_TRANSPORT_TCP_PORT;
+      self->super.logproto = "framed";
+      self->super.sock_type = SOCK_STREAM;
+      self->super.sock_proto = IPPROTO_TCP;
+      self->proxied = TRUE;
+      self->super.transport_name = g_strdup("rfc6587+proxied-tcp");
     }
   else if (strcasecmp(transport, "tls") == 0)
     {
@@ -466,6 +526,27 @@ transport_mapper_syslog_apply_transport(TransportMapper *s, GlobalConfig *cfg)
       self->super.sock_type = SOCK_STREAM;
       self->super.sock_proto = IPPROTO_TCP;
       self->require_tls = TRUE;
+      self->super.transport_name = g_strdup("rfc5425");
+    }
+  else if (strcasecmp(transport, "proxied-tls") == 0)
+    {
+      self->server_port = SYSLOG_TRANSPORT_TCP_PORT;
+      self->super.logproto = "framed";
+      self->super.sock_type = SOCK_STREAM;
+      self->super.sock_proto = IPPROTO_TCP;
+      self->proxied = TRUE;
+      self->require_tls = TRUE;
+      self->super.transport_name = g_strdup("rfc5424+proxied-tls");
+    }
+  else if (strcasecmp(transport, "proxied-tls-passthrough") == 0)
+    {
+      self->server_port = SYSLOG_TRANSPORT_TCP_PORT;
+      self->super.logproto = "framed";
+      self->super.sock_type = SOCK_STREAM;
+      self->super.sock_proto = IPPROTO_TCP;
+      self->proxied = TRUE;
+      self->allow_tls = TRUE;
+      self->super.transport_name = g_strdup("rfc5424+proxied-tls-passthrough");
     }
   else
     {
@@ -475,6 +556,7 @@ transport_mapper_syslog_apply_transport(TransportMapper *s, GlobalConfig *cfg)
       self->server_port = 514;
       self->super.sock_proto = IPPROTO_TCP;
       self->allow_tls = TRUE;
+      self->super.transport_name = g_strdup_printf("rfc5424+%s", self->super.transport);
     }
   g_assert(self->server_port != 0);
 
