@@ -109,14 +109,12 @@ afstomp_dd_set_destination(LogDriver *d, const gchar *destination)
 }
 
 void
-afstomp_dd_set_body(LogDriver *d, const gchar *body)
+afstomp_dd_set_body(LogDriver *d, LogTemplate *body_template)
 {
   STOMPDestDriver *self = (STOMPDestDriver *) d;
-  GlobalConfig *cfg = log_pipe_get_config((LogPipe *)d);
 
-  if (!self->body_template)
-    self->body_template = log_template_new(cfg, NULL);
-  log_template_compile(self->body_template, body, NULL);
+  log_template_unref(self->body_template);
+  self->body_template = body_template;
 }
 
 void
@@ -158,18 +156,20 @@ afstomp_dd_get_template_options(LogDriver *s)
  */
 
 static const gchar *
-afstomp_dd_format_stats_instance(LogThreadedDestDriver *s)
+afstomp_dd_format_stats_key(LogThreadedDestDriver *s, StatsClusterKeyBuilder *kb)
 {
   STOMPDestDriver *self = (STOMPDestDriver *) s;
-  static gchar persist_name[1024];
 
-  if (s->super.super.super.persist_name)
-    g_snprintf(persist_name, sizeof(persist_name), "afstomp,%s", s->super.super.super.persist_name);
-  else
-    g_snprintf(persist_name, sizeof(persist_name), "afstomp,%s,%u,%s", self->host, self->port,
-               self->destination);
+  stats_cluster_key_builder_add_legacy_label(kb, stats_cluster_label("driver", "afstomp"));
+  stats_cluster_key_builder_add_legacy_label(kb, stats_cluster_label("host", self->host));
 
-  return persist_name;
+  gchar num[64];
+  g_snprintf(num, sizeof(num), "%u", self->port);
+  stats_cluster_key_builder_add_legacy_label(kb, stats_cluster_label("port", num));
+
+  stats_cluster_key_builder_add_legacy_label(kb, stats_cluster_label("destination", self->destination));
+
+  return NULL;
 }
 
 static const gchar *
@@ -251,7 +251,7 @@ afstomp_dd_disconnect(LogThreadedDestDriver *s)
 
 /* TODO escape '\0' when passing down the value */
 static gboolean
-afstomp_vp_foreach(const gchar *name, TypeHint type, const gchar *value, gsize value_len,
+afstomp_vp_foreach(const gchar *name, LogMessageValueType type, const gchar *value, gsize value_len,
                    gpointer user_data)
 {
   stomp_frame *frame = (stomp_frame *) (user_data);
@@ -267,7 +267,7 @@ afstomp_set_frame_body(STOMPDestDriver *self, GString *body, stomp_frame *frame,
   if (self->body_template)
     {
       LogTemplateEvalOptions options = {&self->template_options, LTZ_LOCAL,
-                                        self->super.worker.instance.seq_num, NULL
+                                        self->super.worker.instance.seq_num, NULL, LM_VT_STRING
                                        };
       log_template_format(self->body_template, msg, &options, body);
       stomp_frame_set_body(frame, body->str, body->len);
@@ -281,7 +281,6 @@ afstomp_worker_publish(STOMPDestDriver *self, LogMessage *msg)
   GString *body = NULL;
   stomp_frame frame;
   stomp_frame recv_frame;
-  gchar seq_num[16];
 
   if (!self->conn)
     {
@@ -298,11 +297,17 @@ afstomp_worker_publish(STOMPDestDriver *self, LogMessage *msg)
   stomp_frame_add_header(&frame, "destination", self->destination);
   if (self->ack_needed)
     {
-      g_snprintf(seq_num, sizeof(seq_num), "%i", self->super.worker.instance.seq_num);
-      stomp_frame_add_header(&frame, "receipt", seq_num);
+      /*
+       * We check the server's response before sending a new frame to it.
+       * Because of this, we do not need a unique receipt header.
+       *
+       * This changes if multiple workers and/or batching support is introduced.
+       * Make sure to use a unique receipt-id if one of the above gets implemented.
+       */
+      stomp_frame_add_header(&frame, "receipt", "0");
     };
 
-  LogTemplateEvalOptions options = {&self->template_options, LTZ_SEND, self->super.worker.instance.seq_num, NULL};
+  LogTemplateEvalOptions options = {&self->template_options, LTZ_SEND, self->super.worker.instance.seq_num, NULL, LM_VT_STRING};
   value_pairs_foreach(self->vp, afstomp_vp_foreach, msg, &options, &frame);
 
   afstomp_set_frame_body(self, body, &frame, msg);
@@ -392,7 +397,7 @@ afstomp_dd_new(GlobalConfig *cfg)
   self->super.worker.disconnect = afstomp_dd_disconnect;
   self->super.worker.insert = afstomp_worker_insert;
 
-  self->super.format_stats_instance = afstomp_dd_format_stats_instance;
+  self->super.format_stats_key = afstomp_dd_format_stats_key;
   self->super.stats_source = stats_register_type("stomp");
 
   afstomp_dd_set_host((LogDriver *) self, "127.0.0.1");
