@@ -187,10 +187,7 @@ Test(basicfuncs, test_str_funcs)
 #if SYSLOG_NG_ENABLE_IPV6
   assert_template_format("$(dns-resolve-ip 1996::04:30)", "resolved-TEST-host");
 #endif
-  start_grabbing_messages();
   assert_template_format("$(dns-resolve-ip --use-dns=no --dns-cache=yes 123.123.123.123)", "123.123.123.123");
-  assert_grabbed_log_contains("WARNING: With use-dns(no), dns-cache() will be forced to 'no' too!");
-  stop_grabbing_messages();
 
   assert_template_format("$(length $HOST $PID)", "5 5");
   assert_template_format("$(length $HOST)", "5");
@@ -565,6 +562,19 @@ Test(basicfuncs, test_vp_funcs)
   assert_template_format_with_context("$(names)", "");
 }
 
+Test(basicfuncs, test_tag)
+{
+  assert_template_format_value_and_type("$(tag alma)", "1", LM_VT_BOOLEAN);
+  assert_template_format_value_and_type("$(tag korte)", "1", LM_VT_BOOLEAN);
+  assert_template_format_value_and_type("$(tag narancs)", "0", LM_VT_BOOLEAN);
+
+  assert_template_format_value_and_type("$(tag alma true false)", "true", LM_VT_STRING);
+  assert_template_format_value_and_type("$(tag narancs true false)", "false", LM_VT_STRING);
+
+  assert_template_format_value_and_type("$(tags-head alma korte narancs)", "alma", LM_VT_STRING);
+  assert_template_format_value_and_type("$(tags-head narancs alma korte)", "alma", LM_VT_STRING);
+  assert_template_format_value_and_type("$(tags-head narancs banan)", "", LM_VT_NULL);
+}
 
 Test(basicfuncs, test_tfurlencode)
 {
