@@ -61,7 +61,7 @@ extern int correlation_debug;
     CORRELATION_EMPTY = -2,
     CORRELATION_EOF = 0,           /* "end of file"  */
     CORRELATION_error = 256,       /* error  */
-    CORRELATION_UNDEF = 10523,     /* "invalid token"  */
+    CORRELATION_UNDEF = 10531,     /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -73,36 +73,42 @@ extern int correlation_debug;
     LL_CONTEXT_BLOCK_REF = 9,      /* LL_CONTEXT_BLOCK_REF  */
     LL_CONTEXT_BLOCK_CONTENT = 10, /* LL_CONTEXT_BLOCK_CONTENT  */
     LL_CONTEXT_BLOCK_ARG = 11,     /* LL_CONTEXT_BLOCK_ARG  */
-    LL_CONTEXT_PRAGMA = 12,        /* LL_CONTEXT_PRAGMA  */
-    LL_CONTEXT_FORMAT = 13,        /* LL_CONTEXT_FORMAT  */
-    LL_CONTEXT_TEMPLATE_FUNC = 14, /* LL_CONTEXT_TEMPLATE_FUNC  */
-    LL_CONTEXT_INNER_DEST = 15,    /* LL_CONTEXT_INNER_DEST  */
-    LL_CONTEXT_INNER_SRC = 16,     /* LL_CONTEXT_INNER_SRC  */
-    LL_CONTEXT_CLIENT_PROTO = 17,  /* LL_CONTEXT_CLIENT_PROTO  */
-    LL_CONTEXT_SERVER_PROTO = 18,  /* LL_CONTEXT_SERVER_PROTO  */
-    LL_CONTEXT_OPTIONS = 19,       /* LL_CONTEXT_OPTIONS  */
-    LL_CONTEXT_CONFIG = 20,        /* LL_CONTEXT_CONFIG  */
-    LL_CONTEXT_TEMPLATE_REF = 21,  /* LL_CONTEXT_TEMPLATE_REF  */
-    LL_CONTEXT_FILTERX = 22,       /* LL_CONTEXT_FILTERX  */
-    LL_CONTEXT_MAX = 23,           /* LL_CONTEXT_MAX  */
+    LL_CONTEXT_BLOCK_FUNCARG = 12, /* LL_CONTEXT_BLOCK_FUNCARG  */
+    LL_CONTEXT_PRAGMA = 13,        /* LL_CONTEXT_PRAGMA  */
+    LL_CONTEXT_FORMAT = 14,        /* LL_CONTEXT_FORMAT  */
+    LL_CONTEXT_TEMPLATE_FUNC = 15, /* LL_CONTEXT_TEMPLATE_FUNC  */
+    LL_CONTEXT_INNER_DEST = 16,    /* LL_CONTEXT_INNER_DEST  */
+    LL_CONTEXT_INNER_SRC = 17,     /* LL_CONTEXT_INNER_SRC  */
+    LL_CONTEXT_CLIENT_PROTO = 18,  /* LL_CONTEXT_CLIENT_PROTO  */
+    LL_CONTEXT_SERVER_PROTO = 19,  /* LL_CONTEXT_SERVER_PROTO  */
+    LL_CONTEXT_OPTIONS = 20,       /* LL_CONTEXT_OPTIONS  */
+    LL_CONTEXT_CONFIG = 21,        /* LL_CONTEXT_CONFIG  */
+    LL_CONTEXT_TEMPLATE_REF = 22,  /* LL_CONTEXT_TEMPLATE_REF  */
+    LL_CONTEXT_FILTERX = 23,       /* LL_CONTEXT_FILTERX  */
+    LL_CONTEXT_FILTERX_SIMPLE_FUNC = 24, /* LL_CONTEXT_FILTERX_SIMPLE_FUNC  */
+    LL_CONTEXT_FILTERX_ENUM = 25,  /* LL_CONTEXT_FILTERX_ENUM  */
+    LL_CONTEXT_FILTERX_FUNC = 26,  /* LL_CONTEXT_FILTERX_FUNC  */
+    LL_CONTEXT_MAX = 27,           /* LL_CONTEXT_MAX  */
     KW_ASSIGN = 9000,              /* KW_ASSIGN  */
-    KW_OR = 9001,                  /* KW_OR  */
-    KW_AND = 9002,                 /* KW_AND  */
-    KW_STR_EQ = 9003,              /* KW_STR_EQ  */
-    KW_STR_NE = 9004,              /* KW_STR_NE  */
-    KW_TA_EQ = 9005,               /* KW_TA_EQ  */
-    KW_TA_NE = 9006,               /* KW_TA_NE  */
-    KW_TAV_EQ = 9007,              /* KW_TAV_EQ  */
-    KW_TAV_NE = 9008,              /* KW_TAV_NE  */
-    KW_STR_LT = 9009,              /* KW_STR_LT  */
-    KW_STR_LE = 9010,              /* KW_STR_LE  */
-    KW_STR_GE = 9011,              /* KW_STR_GE  */
-    KW_STR_GT = 9012,              /* KW_STR_GT  */
-    KW_TA_LT = 9013,               /* KW_TA_LT  */
-    KW_TA_LE = 9014,               /* KW_TA_LE  */
-    KW_TA_GE = 9015,               /* KW_TA_GE  */
-    KW_TA_GT = 9016,               /* KW_TA_GT  */
-    KW_NOT = 9017,                 /* KW_NOT  */
+    KW_OR = 9010,                  /* KW_OR  */
+    KW_AND = 9020,                 /* KW_AND  */
+    KW_STR_EQ = 9030,              /* KW_STR_EQ  */
+    KW_STR_NE = 9031,              /* KW_STR_NE  */
+    KW_TA_EQ = 9032,               /* KW_TA_EQ  */
+    KW_TA_NE = 9033,               /* KW_TA_NE  */
+    KW_TAV_EQ = 9034,              /* KW_TAV_EQ  */
+    KW_TAV_NE = 9035,              /* KW_TAV_NE  */
+    KW_STR_LT = 9040,              /* KW_STR_LT  */
+    KW_STR_LE = 9041,              /* KW_STR_LE  */
+    KW_STR_GE = 9042,              /* KW_STR_GE  */
+    KW_STR_GT = 9043,              /* KW_STR_GT  */
+    KW_TA_LT = 9044,               /* KW_TA_LT  */
+    KW_TA_LE = 9045,               /* KW_TA_LE  */
+    KW_TA_GE = 9046,               /* KW_TA_GE  */
+    KW_TA_GT = 9047,               /* KW_TA_GT  */
+    KW_PLUS_ASSIGN = 9050,         /* KW_PLUS_ASSIGN  */
+    KW_REGEXP_MATCH = 9060,        /* KW_REGEXP_MATCH  */
+    KW_NOT = 9049,                 /* KW_NOT  */
     KW_SOURCE = 10000,             /* KW_SOURCE  */
     KW_FILTER = 10001,             /* KW_FILTER  */
     KW_PARSER = 10002,             /* KW_PARSER  */
@@ -116,6 +122,7 @@ extern int correlation_debug;
     KW_IF = 10010,                 /* KW_IF  */
     KW_ELSE = 10011,               /* KW_ELSE  */
     KW_ELIF = 10012,               /* KW_ELIF  */
+    KW_FILTERX = 10013,            /* KW_FILTERX  */
     KW_INTERNAL = 10020,           /* KW_INTERNAL  */
     KW_SYSLOG = 10060,             /* KW_SYSLOG  */
     KW_MARK_FREQ = 10071,          /* KW_MARK_FREQ  */
@@ -242,6 +249,7 @@ extern int correlation_debug;
     LL_BLOCK = 10435,              /* LL_BLOCK  */
     LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     LL_TEMPLATE_REF = 10437,       /* LL_TEMPLATE_REF  */
+    LL_MESSAGE_REF = 10438,        /* LL_MESSAGE_REF  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -259,23 +267,24 @@ extern int correlation_debug;
     KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
     KW_RETRIES = 10521,            /* KW_RETRIES  */
     KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_DB_PARSER = 10524,          /* KW_DB_PARSER  */
-    KW_GROUPING_BY = 10525,        /* KW_GROUPING_BY  */
-    KW_INJECT_MODE = 10526,        /* KW_INJECT_MODE  */
-    KW_INHERIT_MODE = 10527,       /* KW_INHERIT_MODE  */
-    KW_TIMEOUT = 10528,            /* KW_TIMEOUT  */
-    KW_TRIGGER = 10529,            /* KW_TRIGGER  */
-    KW_WHERE = 10530,              /* KW_WHERE  */
-    KW_HAVING = 10531,             /* KW_HAVING  */
-    KW_AGGREGATE = 10532,          /* KW_AGGREGATE  */
-    KW_DROP_UNMATCHED = 10533,     /* KW_DROP_UNMATCHED  */
-    KW_FILE = 10534,               /* KW_FILE  */
-    KW_PROGRAM_TEMPLATE = 10535,   /* KW_PROGRAM_TEMPLATE  */
-    KW_MESSAGE_TEMPLATE = 10536,   /* KW_MESSAGE_TEMPLATE  */
-    KW_SORT_KEY = 10537,           /* KW_SORT_KEY  */
-    KW_PREFIX = 10538,             /* KW_PREFIX  */
-    KW_GROUP_LINES = 10539,        /* KW_GROUP_LINES  */
-    KW_LINE_SEPARATOR = 10540      /* KW_LINE_SEPARATOR  */
+    KW_LABELS = 10530,             /* KW_LABELS  */
+    KW_DB_PARSER = 10532,          /* KW_DB_PARSER  */
+    KW_GROUPING_BY = 10533,        /* KW_GROUPING_BY  */
+    KW_INJECT_MODE = 10534,        /* KW_INJECT_MODE  */
+    KW_INHERIT_MODE = 10535,       /* KW_INHERIT_MODE  */
+    KW_TIMEOUT = 10536,            /* KW_TIMEOUT  */
+    KW_TRIGGER = 10537,            /* KW_TRIGGER  */
+    KW_WHERE = 10538,              /* KW_WHERE  */
+    KW_HAVING = 10539,             /* KW_HAVING  */
+    KW_AGGREGATE = 10540,          /* KW_AGGREGATE  */
+    KW_DROP_UNMATCHED = 10541,     /* KW_DROP_UNMATCHED  */
+    KW_FILE = 10542,               /* KW_FILE  */
+    KW_PROGRAM_TEMPLATE = 10543,   /* KW_PROGRAM_TEMPLATE  */
+    KW_MESSAGE_TEMPLATE = 10544,   /* KW_MESSAGE_TEMPLATE  */
+    KW_SORT_KEY = 10545,           /* KW_SORT_KEY  */
+    KW_PREFIX = 10546,             /* KW_PREFIX  */
+    KW_GROUP_LINES = 10547,        /* KW_GROUP_LINES  */
+    KW_LINE_SEPARATOR = 10548      /* KW_LINE_SEPARATOR  */
   };
   typedef enum correlation_tokentype correlation_token_kind_t;
 #endif
@@ -283,7 +292,7 @@ extern int correlation_debug;
 #define CORRELATION_EMPTY -2
 #define CORRELATION_EOF 0
 #define CORRELATION_error 256
-#define CORRELATION_UNDEF 10523
+#define CORRELATION_UNDEF 10531
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -295,36 +304,42 @@ extern int correlation_debug;
 #define LL_CONTEXT_BLOCK_REF 9
 #define LL_CONTEXT_BLOCK_CONTENT 10
 #define LL_CONTEXT_BLOCK_ARG 11
-#define LL_CONTEXT_PRAGMA 12
-#define LL_CONTEXT_FORMAT 13
-#define LL_CONTEXT_TEMPLATE_FUNC 14
-#define LL_CONTEXT_INNER_DEST 15
-#define LL_CONTEXT_INNER_SRC 16
-#define LL_CONTEXT_CLIENT_PROTO 17
-#define LL_CONTEXT_SERVER_PROTO 18
-#define LL_CONTEXT_OPTIONS 19
-#define LL_CONTEXT_CONFIG 20
-#define LL_CONTEXT_TEMPLATE_REF 21
-#define LL_CONTEXT_FILTERX 22
-#define LL_CONTEXT_MAX 23
+#define LL_CONTEXT_BLOCK_FUNCARG 12
+#define LL_CONTEXT_PRAGMA 13
+#define LL_CONTEXT_FORMAT 14
+#define LL_CONTEXT_TEMPLATE_FUNC 15
+#define LL_CONTEXT_INNER_DEST 16
+#define LL_CONTEXT_INNER_SRC 17
+#define LL_CONTEXT_CLIENT_PROTO 18
+#define LL_CONTEXT_SERVER_PROTO 19
+#define LL_CONTEXT_OPTIONS 20
+#define LL_CONTEXT_CONFIG 21
+#define LL_CONTEXT_TEMPLATE_REF 22
+#define LL_CONTEXT_FILTERX 23
+#define LL_CONTEXT_FILTERX_SIMPLE_FUNC 24
+#define LL_CONTEXT_FILTERX_ENUM 25
+#define LL_CONTEXT_FILTERX_FUNC 26
+#define LL_CONTEXT_MAX 27
 #define KW_ASSIGN 9000
-#define KW_OR 9001
-#define KW_AND 9002
-#define KW_STR_EQ 9003
-#define KW_STR_NE 9004
-#define KW_TA_EQ 9005
-#define KW_TA_NE 9006
-#define KW_TAV_EQ 9007
-#define KW_TAV_NE 9008
-#define KW_STR_LT 9009
-#define KW_STR_LE 9010
-#define KW_STR_GE 9011
-#define KW_STR_GT 9012
-#define KW_TA_LT 9013
-#define KW_TA_LE 9014
-#define KW_TA_GE 9015
-#define KW_TA_GT 9016
-#define KW_NOT 9017
+#define KW_OR 9010
+#define KW_AND 9020
+#define KW_STR_EQ 9030
+#define KW_STR_NE 9031
+#define KW_TA_EQ 9032
+#define KW_TA_NE 9033
+#define KW_TAV_EQ 9034
+#define KW_TAV_NE 9035
+#define KW_STR_LT 9040
+#define KW_STR_LE 9041
+#define KW_STR_GE 9042
+#define KW_STR_GT 9043
+#define KW_TA_LT 9044
+#define KW_TA_LE 9045
+#define KW_TA_GE 9046
+#define KW_TA_GT 9047
+#define KW_PLUS_ASSIGN 9050
+#define KW_REGEXP_MATCH 9060
+#define KW_NOT 9049
 #define KW_SOURCE 10000
 #define KW_FILTER 10001
 #define KW_PARSER 10002
@@ -338,6 +353,7 @@ extern int correlation_debug;
 #define KW_IF 10010
 #define KW_ELSE 10011
 #define KW_ELIF 10012
+#define KW_FILTERX 10013
 #define KW_INTERNAL 10020
 #define KW_SYSLOG 10060
 #define KW_MARK_FREQ 10071
@@ -464,6 +480,7 @@ extern int correlation_debug;
 #define LL_BLOCK 10435
 #define LL_PLUGIN 10436
 #define LL_TEMPLATE_REF 10437
+#define LL_MESSAGE_REF 10438
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -481,23 +498,24 @@ extern int correlation_debug;
 #define KW_ON_ERROR 10520
 #define KW_RETRIES 10521
 #define KW_FETCH_NO_DATA_DELAY 10522
-#define KW_DB_PARSER 10524
-#define KW_GROUPING_BY 10525
-#define KW_INJECT_MODE 10526
-#define KW_INHERIT_MODE 10527
-#define KW_TIMEOUT 10528
-#define KW_TRIGGER 10529
-#define KW_WHERE 10530
-#define KW_HAVING 10531
-#define KW_AGGREGATE 10532
-#define KW_DROP_UNMATCHED 10533
-#define KW_FILE 10534
-#define KW_PROGRAM_TEMPLATE 10535
-#define KW_MESSAGE_TEMPLATE 10536
-#define KW_SORT_KEY 10537
-#define KW_PREFIX 10538
-#define KW_GROUP_LINES 10539
-#define KW_LINE_SEPARATOR 10540
+#define KW_LABELS 10530
+#define KW_DB_PARSER 10532
+#define KW_GROUPING_BY 10533
+#define KW_INJECT_MODE 10534
+#define KW_INHERIT_MODE 10535
+#define KW_TIMEOUT 10536
+#define KW_TRIGGER 10537
+#define KW_WHERE 10538
+#define KW_HAVING 10539
+#define KW_AGGREGATE 10540
+#define KW_DROP_UNMATCHED 10541
+#define KW_FILE 10542
+#define KW_PROGRAM_TEMPLATE 10543
+#define KW_MESSAGE_TEMPLATE 10544
+#define KW_SORT_KEY 10545
+#define KW_PREFIX 10546
+#define KW_GROUP_LINES 10547
+#define KW_LINE_SEPARATOR 10548
 
 /* Value type.  */
 #if ! defined CORRELATION_STYPE && ! defined CORRELATION_STYPE_IS_DECLARED
