@@ -216,7 +216,7 @@ _grab_dirlock(const gchar *dir, gint *fd)
 
   g_mutex_lock(&filename_lock);
 
-  *fd = open(dirlock_file_path, O_RDONLY | O_CREAT, 0600);
+  *fd = open(dirlock_file_path, O_RDWR | O_CREAT, 0600);
   if (*fd < 0)
     {
       msg_error("Failed to open disk-buffer dirlock file",
@@ -458,7 +458,7 @@ _maybe_truncate_file(QDisk *self, gint64 expected_size)
             evt_tag_int("fd", self->fd));
 }
 
-#if !SYSLOG_NG_HAVE_POSIX_FALLOCATE
+#ifndef SYSLOG_NG_HAVE_POSIX_FALLOCATE
 static gint
 _compat_preallocate(int fd, off_t offset, off_t len)
 {
@@ -494,7 +494,7 @@ _preallocate_qdisk_file(QDisk *self, off_t size)
 
   gint result;
 
-#if SYSLOG_NG_HAVE_POSIX_FALLOCATE
+#ifdef SYSLOG_NG_HAVE_POSIX_FALLOCATE
   result = posix_fallocate(self->fd, QDISK_RESERVED_SPACE, size - QDISK_RESERVED_SPACE);
 #else
   result = _compat_preallocate(self->fd, QDISK_RESERVED_SPACE, size - QDISK_RESERVED_SPACE);
@@ -882,6 +882,25 @@ qdisk_get_next_head_position(QDisk *self)
   return next_read_head_position;
 }
 
+gboolean
+qdisk_peek_head(QDisk *self, GString *record)
+{
+  if (self->hdr->read_head == self->hdr->write_head)
+    return FALSE;
+
+  if (self->hdr->read_head > self->hdr->write_head)
+    self->hdr->read_head = _correct_position_if_max_size_is_reached(self, self->hdr->read_head);
+
+  guint32 record_length;
+  if (!_try_reading_record_length(self, self->hdr->read_head, &record_length))
+    return FALSE;
+
+  if (!_read_record_from_disk(self, record, record_length))
+    return FALSE;
+
+  return TRUE;
+}
+
 gboolean
 qdisk_pop_head(QDisk *self, GString *record)
 {
