@@ -187,14 +187,16 @@ threaded_feed(gpointer args)
   gint i;
   LogPathOptions path_options = LOG_PATH_OPTIONS_INIT;
   LogMessage *msg, *tmpl;
-  GTimeVal start, end;
+  struct timespec start, end;
   glong diff;
 
+  iv_init();
+
   /* emulate main loop for LogQueue */
   main_loop_worker_thread_start(MLW_ASYNC_WORKER);
 
   tmpl = log_msg_new_empty();
-  g_get_current_time(&start);
+  clock_gettime(CLOCK_MONOTONIC, &start);
   for (i = 0; i < MESSAGES_PER_FEEDER; i++)
     {
       main_loop_worker_run_gc();
@@ -208,13 +210,14 @@ threaded_feed(gpointer args)
         main_loop_worker_invoke_batch_callbacks();
     }
   main_loop_worker_invoke_batch_callbacks();
-  g_get_current_time(&end);
-  diff = g_time_val_diff(&end, &start);
+  clock_gettime(CLOCK_MONOTONIC, &end);
+  diff = timespec_diff_usec(&end, &start);
   g_mutex_lock(&tlock);
   sum_time += diff;
   g_mutex_unlock(&tlock);
   main_loop_worker_thread_stop();
   log_msg_unref(tmpl);
+  iv_deinit();
   return NULL;
 }
 
@@ -225,6 +228,8 @@ threaded_consume(gpointer st)
   LogPathOptions path_options = LOG_PATH_OPTIONS_INIT;
   gint i;
 
+  iv_init();
+
   /* just to make sure time is properly cached */
   main_loop_worker_thread_start(MLW_ASYNC_WORKER);
 
@@ -260,6 +265,7 @@ threaded_consume(gpointer st)
 
   main_loop_worker_thread_stop();
 
+  iv_deinit();
   return NULL;
 }
 
