@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2023 Attila Szakacs
+ * Copyright (c) 2024 Axoflow
+ * Copyright (c) 2023-2024 Attila Szakacs <attila.szakacs@axoflow.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -23,15 +24,18 @@
 #ifndef OTEL_DEST_HPP
 #define OTEL_DEST_HPP
 
-#include <grpcpp/server.h>
+#include "otel-dest.h"
 
 #include "compat/cpp-start.h"
 #include "logthrdest/logthrdestdrv.h"
 #include "compat/cpp-end.h"
 
-#include "otel-dest.h"
 #include "credentials/grpc-credentials-builder.hpp"
+#include "metrics/grpc-metrics.hpp"
 
+#include <grpcpp/server.h>
+
+#include <list>
 
 namespace syslogng {
 namespace grpc {
@@ -46,6 +50,17 @@ public:
   void set_url(const char *url);
   const std::string &get_url() const;
 
+  void set_compression(bool enable);
+  bool get_compression() const;
+
+  void set_batch_bytes(size_t bytes);
+  size_t get_batch_bytes() const;
+
+  void add_extra_channel_arg(std::string name, long value);
+  void add_extra_channel_arg(std::string name, std::string value);
+
+  void add_header(std::string name, std::string value);
+
   virtual bool init();
   virtual bool deinit();
   virtual const char *format_stats_key(StatsClusterKeyBuilder *kb);
@@ -53,6 +68,7 @@ public:
   virtual LogThreadedDestWorker *construct_worker(int worker_index);
 
   GrpcClientCredentialsBuilderW *get_credentials_builder_wrapper();
+
 public:
   syslogng::grpc::ClientCredentialsBuilder credentials_builder;
 
@@ -60,7 +76,13 @@ protected:
   friend class DestWorker;
   OtelDestDriver *super;
   std::string url;
+  bool compression;
+  size_t batch_bytes;
+  std::list<std::pair<std::string, long>> int_extra_channel_args;
+  std::list<std::pair<std::string, std::string>> string_extra_channel_args;
+  std::list<std::pair<std::string, std::string>> headers;
   GrpcClientCredentialsBuilderW credentials_builder_wrapper;
+  DestDriverMetrics metrics;
 };
 
 }
