@@ -54,6 +54,7 @@ class AsyncServiceCallInterface
 {
 public:
   virtual void Proceed(bool ok) = 0;
+  virtual ~AsyncServiceCallInterface() = default;
 };
 
 template <class S, class Req, class Res>
@@ -63,14 +64,14 @@ public:
   void Proceed(bool ok) override;
 
 public:
-  AsyncServiceCall(SourceDriver &driver_, S *service_, ::grpc::ServerCompletionQueue *cq_)
-    : driver(driver_), service(service_), responder(&ctx), cq(cq_), status(PROCESS)
+  AsyncServiceCall(SourceWorker &worker_, S *service_, ::grpc::ServerCompletionQueue *cq_)
+    : worker(worker_), service(service_), responder(&ctx), cq(cq_), status(PROCESS)
   {
     service->RequestExport(&ctx, &request, &responder, cq, cq, this);
   }
 
 private:
-  SourceDriver &driver;
+  SourceWorker &worker;
   S *service;
   ::grpc::ServerAsyncResponseWriter<Res> responder;
   Req request;
@@ -96,10 +97,13 @@ syslogng::grpc::otel::TraceServiceCall::Proceed(bool ok)
       return;
     }
 
-  new TraceServiceCall(driver, service, cq);
+  if (!worker.super->super.under_termination)
+    new TraceServiceCall(worker, service, cq);
 
   ::grpc::Status response_status = ::grpc::Status::OK;
 
+  int msgs_in_fetch_round = 0;
+
   for (const ResourceSpans &resource_spans : request.resource_spans())
     {
       const Resource &resource = resource_spans.resource();
@@ -112,19 +116,31 @@ syslogng::grpc::otel::TraceServiceCall::Proceed(bool ok)
 
           for (const Span &span : scope_spans.spans())
             {
-              LogMessage *msg = log_msg_new_empty();
-              ProtobufParser::store_raw_metadata(msg, ctx.peer(), resource, resource_spans_schema_url, scope,
-                                                 scope_spans_schema_url);
-              ProtobufParser::store_raw(msg, span);
-              if (!driver.post(msg))
+              if (worker.super->super.under_termination)
                 {
                   response_status = ::grpc::Status(::grpc::StatusCode::UNAVAILABLE, "Server is unavailable");
                   break;
                 }
+
+              LogMessage *msg = log_msg_new_empty();
+              ProtobufParser::store_raw_metadata(msg, ctx.peer(), resource, resource_spans_schema_url, scope,
+                                                 scope_spans_schema_url);
+              ProtobufParser::store_raw(msg, span);
+              worker.post(msg);
+
+              msgs_in_fetch_round++;
+              if (msgs_in_fetch_round == worker.driver.fetch_limit)
+                {
+                  log_threaded_source_worker_close_batch(&worker.super->super);
+                  msgs_in_fetch_round = 0;
+                }
             }
         }
     }
 
+  if (msgs_in_fetch_round != 0)
+    log_threaded_source_worker_close_batch(&worker.super->super);
+
   status = FINISH;
   responder.Finish(response, response_status, this);
 }
@@ -138,10 +154,13 @@ syslogng::grpc::otel::LogsServiceCall::Proceed(bool ok)
       return;
     }
 
-  new LogsServiceCall(driver, service, cq);
+  if (!worker.super->super.under_termination)
+    new LogsServiceCall(worker, service, cq);
 
   ::grpc::Status response_status = ::grpc::Status::OK;
 
+  int msgs_in_fetch_round = 0;
+
   for (const ResourceLogs &resource_logs : request.resource_logs())
     {
       const Resource &resource = resource_logs.resource();
@@ -154,6 +173,12 @@ syslogng::grpc::otel::LogsServiceCall::Proceed(bool ok)
 
           for (const LogRecord &log_record : scope_logs.log_records())
             {
+              if (worker.super->super.under_termination)
+                {
+                  response_status = ::grpc::Status(::grpc::StatusCode::UNAVAILABLE, "Server is unavailable");
+                  break;
+                }
+
               LogMessage *msg = log_msg_new_empty();
               if (ProtobufParser::is_syslog_ng_log_record(resource, resource_logs_schema_url, scope,
                                                           scope_logs_schema_url))
@@ -166,15 +191,21 @@ syslogng::grpc::otel::LogsServiceCall::Proceed(bool ok)
                                                      scope_logs_schema_url);
                   ProtobufParser::store_raw(msg, log_record);
                 }
-              if (!driver.post(msg))
+              worker.post(msg);
+
+              msgs_in_fetch_round++;
+              if (msgs_in_fetch_round == worker.driver.fetch_limit)
                 {
-                  response_status = ::grpc::Status(::grpc::StatusCode::UNAVAILABLE, "Server is unavailable");
-                  break;
+                  log_threaded_source_worker_close_batch(&worker.super->super);
+                  msgs_in_fetch_round = 0;
                 }
             }
         }
     }
 
+  if (msgs_in_fetch_round != 0)
+    log_threaded_source_worker_close_batch(&worker.super->super);
+
   status = FINISH;
   responder.Finish(response, response_status, this);
 }
@@ -188,10 +219,13 @@ syslogng::grpc::otel::MetricsServiceCall::Proceed(bool ok)
       return;
     }
 
-  new MetricsServiceCall(driver, service, cq);
+  if (!worker.super->super.under_termination)
+    new MetricsServiceCall(worker, service, cq);
 
   ::grpc::Status response_status = ::grpc::Status::OK;
 
+  int msgs_in_fetch_round = 0;
+
   for (const ResourceMetrics &resource_metrics : request.resource_metrics())
     {
       const Resource &resource = resource_metrics.resource();
@@ -204,19 +238,31 @@ syslogng::grpc::otel::MetricsServiceCall::Proceed(bool ok)
 
           for (const Metric &metric : scope_metrics.metrics())
             {
-              LogMessage *msg = log_msg_new_empty();
-              ProtobufParser::store_raw_metadata(msg, ctx.peer(), resource, resource_metrics_schema_url, scope,
-                                                 scope_metrics_schema_url);
-              ProtobufParser::store_raw(msg, metric);
-              if (!driver.post(msg))
+              if (worker.super->super.under_termination)
                 {
                   response_status = ::grpc::Status(::grpc::StatusCode::UNAVAILABLE, "Server is unavailable");
                   break;
                 }
+
+              LogMessage *msg = log_msg_new_empty();
+              ProtobufParser::store_raw_metadata(msg, ctx.peer(), resource, resource_metrics_schema_url, scope,
+                                                 scope_metrics_schema_url);
+              ProtobufParser::store_raw(msg, metric);
+              worker.post(msg);
+
+              msgs_in_fetch_round++;
+              if (msgs_in_fetch_round == worker.driver.fetch_limit)
+                {
+                  log_threaded_source_worker_close_batch(&worker.super->super);
+                  msgs_in_fetch_round = 0;
+                }
             }
         }
     }
 
+  if (msgs_in_fetch_round != 0)
+    log_threaded_source_worker_close_batch(&worker.super->super);
+
   status = FINISH;
   responder.Finish(response, response_status, this);
 }
