@@ -20,11 +20,6 @@
  *
  */
 
-#include <string>
-
-#include <grpcpp/grpcpp.h>
-#include <grpcpp/server_builder.h>
-
 #include "otel-source.hpp"
 #include "otel-source-services.hpp"
 #include "credentials/grpc-credentials-builder.hpp"
@@ -33,7 +28,13 @@
 #include "messages.h"
 #include "compat/cpp-end.h"
 
+#include <string>
+
+#include <grpcpp/grpcpp.h>
+#include <grpcpp/server_builder.h>
+
 #define get_SourceDriver(s) (((OtelSourceDriver *) s)->cpp)
+#define get_SourceWorker(s) (((OtelSourceWorker *) s)->cpp)
 
 using namespace syslogng::grpc::otel;
 
@@ -45,51 +46,11 @@ syslogng::grpc::otel::SourceDriver::SourceDriver(OtelSourceDriver *s)
   credentials_builder_wrapper.self = &credentials_builder;
 }
 
-void
-syslogng::grpc::otel::SourceDriver::run()
-{
-  std::string address = std::string("[::]:").append(std::to_string(port));
-
-  ::grpc::EnableDefaultHealthCheckService(true);
-
-  ::grpc::ServerBuilder builder;
-  builder.AddListeningPort(address, credentials_builder.build());
-
-  TraceService::AsyncService trace_service;
-  LogsService::AsyncService logs_service;
-  MetricsService::AsyncService metrics_service;
-
-  builder.RegisterService(&trace_service);
-  builder.RegisterService(&logs_service);
-  builder.RegisterService(&metrics_service);
-
-  cq = builder.AddCompletionQueue();
-  server = builder.BuildAndStart();
-  if (!server)
-    {
-      msg_error("Failed to start OpenTelemetry server", evt_tag_int("port", port));
-      return;
-    }
-
-  msg_info("OpenTelemetry server accepting connections", evt_tag_int("port", port));
-
-  new TraceServiceCall(*this, &trace_service, cq.get());
-  new LogsServiceCall(*this, &logs_service, cq.get());
-  new MetricsServiceCall(*this, &metrics_service, cq.get());
-
-  void *tag;
-  bool ok;
-  while (cq->Next(&tag, &ok))
-    {
-      static_cast<AsyncServiceCallInterface *>(tag)->Proceed(ok);
-    }
-}
-
 void
 syslogng::grpc::otel::SourceDriver::request_exit()
 {
-  server->Shutdown();
-  cq->Shutdown();
+  msg_debug("Shutting down OpenTelemetry server", evt_tag_int("port", port));
+  server->Shutdown(std::chrono::system_clock::now() + std::chrono::seconds(30));
 }
 
 void
@@ -123,23 +84,75 @@ syslogng::grpc::otel::SourceDriver::init()
   if (!credentials_builder.validate())
     return FALSE;
 
+  std::string address = std::string("[::]:").append(std::to_string(port));
+
+  ::grpc::EnableDefaultHealthCheckService(true);
+
+  ::grpc::ServerBuilder builder;
+  builder.AddListeningPort(address, credentials_builder.build());
+
+  for (auto nv : int_extra_channel_args)
+    builder.AddChannelArgument(nv.first, nv.second);
+  for (auto nv : string_extra_channel_args)
+    builder.AddChannelArgument(nv.first, nv.second);
+
+  trace_service = std::make_unique<TraceService::AsyncService>();
+  logs_service = std::make_unique<LogsService::AsyncService>();
+  metrics_service = std::make_unique<MetricsService::AsyncService>();
+
+  builder.RegisterService(trace_service.get());
+  builder.RegisterService(logs_service.get());
+  builder.RegisterService(metrics_service.get());
+
+  for (int i = 0; i < super->super.num_workers; i++)
+    cqs.push_back(builder.AddCompletionQueue());
+
+  server = builder.BuildAndStart();
+  if (!server)
+    {
+      msg_error("Failed to start OpenTelemetry server", evt_tag_int("port", port));
+      return false;
+    }
+
+  msg_info("OpenTelemetry server accepting connections", evt_tag_int("port", port));
+
+  if (fetch_limit == -1)
+    {
+      if (super->super.worker_options.super.init_window_size != -1)
+        fetch_limit = super->super.worker_options.super.init_window_size / super->super.num_workers;
+      else
+        fetch_limit = 100;
+    }
+
+  /*
+   * syslog-ng-otlp(): the original HOST is always kept
+   * opentelemetry(): there is no parsing in this source, HOST always falls back to saddr (LogSource)
+   */
+  super->super.worker_options.super.keep_hostname = TRUE;
+
   return log_threaded_source_driver_init_method(&super->super.super.super.super);
 }
 
 gboolean
 syslogng::grpc::otel::SourceDriver::deinit()
 {
+  trace_service = nullptr;
+  logs_service = nullptr;
+  metrics_service = nullptr;
+
   return log_threaded_source_driver_deinit_method(&super->super.super.super.super);
 }
 
-bool
-syslogng::grpc::otel::SourceDriver::post(LogMessage *msg)
+void
+SourceDriver::add_extra_channel_arg(std::string name, long value)
 {
-  if (!log_threaded_source_free_to_send(&super->super))
-    return false;
+  int_extra_channel_args.push_back(std::pair<std::string, long> {name, value});
+}
 
-  log_threaded_source_post(&super->super, msg);
-  return true;
+void
+SourceDriver::add_extra_channel_arg(std::string name, std::string value)
+{
+  string_extra_channel_args.push_back(std::pair<std::string, std::string> {name, value});
 }
 
 GrpcServerCredentialsBuilderW *
@@ -148,6 +161,49 @@ SourceDriver::get_credentials_builder_wrapper()
   return &credentials_builder_wrapper;
 }
 
+SourceWorker::SourceWorker(OtelSourceWorker *s, SourceDriver &d)
+  : super(s), driver(d)
+{
+  cq = std::move(driver.cqs.front());
+  driver.cqs.pop_front();
+}
+
+void
+syslogng::grpc::otel::SourceWorker::run()
+{
+  /* Proceed() will immediately create a new ServiceCall,
+   * so creating 1 ServiceCall here results in 2 concurrent requests.
+   *
+   * Because of this we should create (concurrent_requests - 1) ServiceCalls here.
+   */
+  for (int i = 0; i < driver.concurrent_requests - 1; i++)
+    {
+      new TraceServiceCall(*this, driver.trace_service.get(), cq.get());
+      new LogsServiceCall(*this, driver.logs_service.get(), cq.get());
+      new MetricsServiceCall(*this, driver.metrics_service.get(), cq.get());
+    }
+
+  void *tag;
+  bool ok;
+  while (cq->Next(&tag, &ok))
+    {
+      static_cast<AsyncServiceCallInterface *>(tag)->Proceed(ok);
+    }
+}
+
+void
+syslogng::grpc::otel::SourceWorker::request_exit()
+{
+  driver.request_exit();
+  cq->Shutdown();
+}
+
+void
+SourceWorker::post(LogMessage *msg)
+{
+  log_threaded_source_worker_blocking_post(&super->super, msg);
+}
+
 /* Config setters */
 
 void
@@ -156,6 +212,30 @@ otel_sd_set_port(LogDriver *s, guint64 port)
   get_SourceDriver(s)->port = port;
 }
 
+void
+otel_sd_set_fetch_limit(LogDriver *s, gint fetch_limit)
+{
+  get_SourceDriver(s)->fetch_limit = fetch_limit;
+}
+
+void
+otel_sd_set_concurrent_requests(LogDriver *s, gint concurrent_requests)
+{
+  get_SourceDriver(s)->concurrent_requests = concurrent_requests;
+}
+
+void
+otel_sd_add_int_channel_arg(LogDriver *s, const gchar *name, gint64 value)
+{
+  get_SourceDriver(s)->add_extra_channel_arg(name, value);
+}
+
+void
+otel_sd_add_string_channel_arg(LogDriver *s, const gchar *name, const gchar *value)
+{
+  get_SourceDriver(s)->add_extra_channel_arg(name, value);
+}
+
 GrpcServerCredentialsBuilderW *
 otel_sd_get_credentials_builder(LogDriver *s)
 {
@@ -165,15 +245,37 @@ otel_sd_get_credentials_builder(LogDriver *s)
 /* C Wrappers */
 
 static void
-_run(LogThreadedSourceDriver *s)
+_worker_free(LogPipe *s)
 {
-  get_SourceDriver(s)->run();
+  delete get_SourceWorker(s);
+  log_threaded_source_worker_free(s);
 }
 
 static void
-_request_exit(LogThreadedSourceDriver *s)
+_worker_run(LogThreadedSourceWorker *s)
 {
-  get_SourceDriver(s)->request_exit();
+  get_SourceWorker(s)->run();
+}
+
+static void
+_worker_request_exit(LogThreadedSourceWorker *s)
+{
+  get_SourceWorker(s)->request_exit();
+}
+
+static LogThreadedSourceWorker *
+_construct_worker(LogThreadedSourceDriver *s, gint worker_index)
+{
+  OtelSourceWorker *worker = g_new0(OtelSourceWorker, 1);
+  log_threaded_source_worker_init_instance(&worker->super, s, worker_index);
+
+  worker->cpp = new SourceWorker(worker, *get_SourceDriver(s));
+
+  worker->super.run = _worker_run;
+  worker->super.request_exit = _worker_request_exit;
+  worker->super.super.super.free_fn = _worker_free;
+
+  return &worker->super;
 }
 
 static void
@@ -212,7 +314,7 @@ otel_sd_new(GlobalConfig *cfg)
 {
   OtelSourceDriver *s = g_new0(OtelSourceDriver, 1);
   log_threaded_source_driver_init_instance(&s->super, cfg);
-
+  log_threaded_source_driver_set_transport_name(&s->super, "otlp");
   s->cpp = new syslogng::grpc::otel::SourceDriver(s);
 
   s->super.super.super.super.init = _init;
@@ -222,8 +324,9 @@ otel_sd_new(GlobalConfig *cfg)
 
   s->super.worker_options.super.stats_source = stats_register_type("opentelemetry");
   s->super.format_stats_key = _format_stats_key;
-  s->super.run = _run;
-  s->super.request_exit = _request_exit;
+  s->super.worker_construct = _construct_worker;
+
+  s->super.auto_close_batches = FALSE;
 
   return &s->super.super.super;
 }
