@@ -23,15 +23,23 @@
 #ifndef OTEL_SOURCE_H
 #define OTEL_SOURCE_H
 
+#include "syslog-ng.h"
+
 #include "compat/cpp-start.h"
 
 #include "driver.h"
 #include "credentials/grpc-credentials-builder.h"
 
+typedef struct OtelSourceWorker_ OtelSourceWorker;
 typedef struct OtelSourceDriver_ OtelSourceDriver;
 
 LogDriver *otel_sd_new(GlobalConfig *cfg);
 void otel_sd_set_port(LogDriver *s, guint64 port);
+void otel_sd_set_fetch_limit(LogDriver *s, gint fetch_limit);
+void otel_sd_set_concurrent_requests(LogDriver *s, gint concurrent_requests);
+void otel_sd_add_int_channel_arg(LogDriver *s, const gchar *name, gint64 value);
+void otel_sd_add_string_channel_arg(LogDriver *s, const gchar *name, const gchar *value);
+
 GrpcServerCredentialsBuilderW *otel_sd_get_credentials_builder(LogDriver *s);
 
 #include "compat/cpp-end.h"
