@@ -23,6 +23,7 @@
 
 #include "otel-protobuf-formatter.hpp"
 #include "otel-protobuf-parser.hpp"
+#include "otel-logmsg-handles.hpp"
 
 #include "compat/cpp-start.h"
 #include "apphook.h"
@@ -61,7 +62,7 @@ static void
 _assert_dummy_resource_and_scope(const Resource &resource, const std::string &resource_schema_url,
                                  const InstrumentationScope &scope, const std::string &scope_schema_url)
 {
-  cr_assert_eq(resource.attributes_size(), 1, "%d");
+  cr_assert_eq(resource.attributes_size(), 1);
   cr_assert_str_eq(resource.attributes(0).key().c_str(), "attr_0");
   cr_assert_str_eq(resource.attributes(0).value().string_value().c_str(), "val_0");
   cr_assert_eq(resource.dropped_attributes_count(), 1);
@@ -157,7 +158,7 @@ _log_record_tc_asserts(const LogRecord &log_record)
   cr_assert_str_eq(log_record.body().string_value().c_str(), "string_body");
 
   auto &attributes = log_record.attributes();
-  cr_assert_eq(log_record.attributes_size(), 8);
+  cr_assert_eq(log_record.attributes_size(), 9);
   cr_assert_str_eq(attributes.at(0).key().c_str(), "a_string_key");
   cr_assert_eq(attributes.at(0).value().value_case(), AnyValue::kStringValue);
   cr_assert_str_eq(attributes.at(0).value().string_value().c_str(), "string");
@@ -188,6 +189,13 @@ _log_record_tc_asserts(const LogRecord &log_record)
   cr_assert_eq(attributes.at(7).value().array_value().values_size(), 2);
   cr_assert_eq(attributes.at(7).value().array_value().values().at(0).int_value(), 1337);
   cr_assert_eq(attributes.at(7).value().array_value().values().at(1).int_value(), 7331);
+  cr_assert_str_eq(attributes.at(8).key().c_str(), "i_list_key");
+  cr_assert_eq(attributes.at(8).value().value_case(), AnyValue::kArrayValue);
+  cr_assert_eq(attributes.at(8).value().array_value().values_size(), 4);
+  cr_assert_str_eq(attributes.at(8).value().array_value().values().at(0).string_value().c_str(), "a,");
+  cr_assert_str_eq(attributes.at(8).value().array_value().values().at(1).string_value().c_str(), "b");
+  cr_assert_str_eq(attributes.at(8).value().array_value().values().at(2).string_value().c_str(), "c,");
+  cr_assert_str_eq(attributes.at(8).value().array_value().values().at(3).string_value().c_str(), "d");
 
   cr_assert_eq(log_record.dropped_attributes_count(), 11);
   cr_assert_eq(log_record.flags(), 22);
@@ -239,6 +247,8 @@ Test(otel_protobuf_formatter, log_record)
                                       any_value_array_serialized.c_str(), any_value_array_serialized.length(),
                                       LM_VT_PROTOBUF);
 
+  log_msg_set_value_by_name_with_type(msg, ".otel.log.attributes.i_list_key", "\"a,\",b,\"c,\",d", -1, LM_VT_LIST);
+
   log_msg_set_value_by_name_with_type(msg, ".otel.log.dropped_attributes_count", "11", -1, LM_VT_INTEGER);
   log_msg_set_value_by_name_with_type(msg, ".otel.log.flags", "22", -1, LM_VT_INTEGER);
   log_msg_set_value_by_name_with_type(msg, ".otel.log.trace_id", "\0\1\2\3\4\5\6\7\0\1\2\3\4\5\6\7", 16, LM_VT_BYTES);
@@ -530,8 +540,8 @@ _metric_histogram_tc_asserts(const Metric &metric)
   cr_assert_eq(data_point_0.bucket_counts().at(0), 22);
   cr_assert_eq(data_point_0.bucket_counts().at(1), 33);
   cr_assert_eq(data_point_0.explicit_bounds_size(), 2);
-  cr_assert_eq(data_point_0.explicit_bounds().at(0), 44.44);
-  cr_assert_eq(data_point_0.explicit_bounds().at(1), 55.55);
+  cr_assert_float_eq(data_point_0.explicit_bounds().at(0), 44.44, std::numeric_limits<double>::epsilon());
+  cr_assert_float_eq(data_point_0.explicit_bounds().at(1), 55.55, std::numeric_limits<double>::epsilon());
   cr_assert_eq(data_point_0.exemplars_size(), 1);
   cr_assert_eq(data_point_0.exemplars().at(0).filtered_attributes_size(), 1);
   cr_assert_str_eq(data_point_0.exemplars().at(0).filtered_attributes().at(0).key().c_str(), "a_0");
@@ -991,6 +1001,7 @@ setup(void)
 {
   app_startup();
   configuration = cfg_new_snippet();
+  otel_logmsg_handles_global_init();
 }
 
 void
