@@ -29,6 +29,7 @@
 #include "http-loadbalancer.h"
 #include "http-curl-header-list.h"
 #include "compression.h"
+#include "metrics/dyn-metrics-store.h"
 
 typedef struct _HTTPDestinationWorker
 {
@@ -39,6 +40,14 @@ typedef struct _HTTPDestinationWorker
   GString *request_body_compressed;
   Compressor *compressor;
   List *request_headers;
+  GString *url_buffer;
+  LogMessage *msg_for_templated_url;
+
+  struct
+  {
+    DynMetricsStore *cache;
+    gchar requests_response_code_str_buffer[4];
+  } metrics;
 } HTTPDestinationWorker;
 
 LogThreadedResult default_map_http_status_to_worker_status(HTTPDestinationWorker *self, const gchar *url,
