@@ -33,8 +33,14 @@ http_dd_insert_response_handler(LogDriver *d, HttpResponseHandler *response_hand
   http_response_handlers_insert(self->response_handlers, response_handler);
 }
 
-void
-http_dd_set_urls(LogDriver *d, GList *url_strings)
+static gboolean
+_is_url_templated(const gchar *url)
+{
+  return strchr(url, '$') != NULL;
+}
+
+gboolean
+http_dd_set_urls(LogDriver *d, GList *url_strings, GError **error)
 {
   HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
 
@@ -42,14 +48,28 @@ http_dd_set_urls(LogDriver *d, GList *url_strings)
   for (GList *l = url_strings; l; l = l->next)
     {
       const gchar *url_string = (const gchar *) l->data;
-      gchar **urls = g_strsplit(url_string, " ", -1);
 
+      if (_is_url_templated(url_string))
+        {
+          /* Templated URLs might contain spaces, so we should handle the string as one URL. */
+          if (!http_load_balancer_add_target(self->load_balancer, url_string, error))
+            return FALSE;
+          continue;
+        }
+
+      gchar **urls = g_strsplit(url_string, " ", -1);
       for (gint url = 0; urls[url]; url++)
         {
-          http_load_balancer_add_target(self->load_balancer, urls[url]);
+          if (!http_load_balancer_add_target(self->load_balancer, urls[url], error))
+            {
+              g_strfreev(urls);
+              return FALSE;
+            }
         }
       g_strfreev(urls);
     }
+
+  return TRUE;
 }
 
 void
@@ -99,7 +119,7 @@ http_dd_set_method(LogDriver *d, const gchar *method)
     self->method_type = METHOD_TYPE_PUT;
   else
     {
-      msg_warning("Unsupported method is set(Only POST and PUT are supported), default method POST will be used",
+      msg_warning("http: Unsupported method is set(Only POST and PUT are supported), default method POST will be used",
                   evt_tag_str("method", method));
       self->method_type = METHOD_TYPE_POST;
     }
@@ -203,7 +223,7 @@ http_dd_set_ssl_version(LogDriver *d, const gchar *value)
 {
   HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
 
-  if (strcmp(value, "default") == 0)
+  if (strcasecmp(value, "default") == 0)
     {
       /*
        * Negotiate the version based on what the remote server supports.
@@ -213,45 +233,45 @@ http_dd_set_ssl_version(LogDriver *d, const gchar *value)
       self->ssl_version = CURL_SSLVERSION_DEFAULT;
 
     }
-  else if (strcmp(value, "tlsv1") == 0)
+  else if (strcasecmp(value, "tlsv1") == 0)
     {
       /* TLS 1.x */
       self->ssl_version = CURL_SSLVERSION_TLSv1;
     }
-  else if (strcmp(value, "sslv2") == 0)
+  else if (strcasecmp(value, "sslv2") == 0)
     {
       /* SSL 2 only */
       self->ssl_version = CURL_SSLVERSION_SSLv2;
 
     }
-  else if (strcmp(value, "sslv3") == 0)
+  else if (strcasecmp(value, "sslv3") == 0)
     {
       /* SSL 3 only */
       self->ssl_version = CURL_SSLVERSION_SSLv3;
     }
 #if SYSLOG_NG_HAVE_DECL_CURL_SSLVERSION_TLSV1_0
-  else if (strcmp(value, "tlsv1_0") == 0)
+  else if (strcasecmp(value, "tlsv1_0") == 0)
     {
       /* TLS 1.0 only */
       self->ssl_version = CURL_SSLVERSION_TLSv1_0;
     }
 #endif
 #if SYSLOG_NG_HAVE_DECL_CURL_SSLVERSION_TLSV1_1
-  else if (strcmp(value, "tlsv1_1") == 0)
+  else if (strcasecmp(value, "tlsv1_1") == 0)
     {
       /* TLS 1.1 only */
       self->ssl_version = CURL_SSLVERSION_TLSv1_1;
     }
 #endif
 #if SYSLOG_NG_HAVE_DECL_CURL_SSLVERSION_TLSV1_2
-  else if (strcmp(value, "tlsv1_2") == 0)
+  else if (strcasecmp(value, "tlsv1_2") == 0)
     {
       /* TLS 1.2 only */
       self->ssl_version = CURL_SSLVERSION_TLSv1_2;
     }
 #endif
 #if SYSLOG_NG_HAVE_DECL_CURL_SSLVERSION_TLSV1_3
-  else if (strcmp(value, "tlsv1_3") == 0)
+  else if (strcasecmp(value, "tlsv1_3") == 0)
     {
       /* TLS 1.3 only */
       self->ssl_version = CURL_SSLVERSION_TLSv1_3;
@@ -270,35 +290,28 @@ http_dd_set_accept_encoding(LogDriver *d, const gchar *encoding)
 
   if (self->accept_encoding != NULL)
     g_string_free(self->accept_encoding, TRUE);
-
+#if SYSLOG_NG_HTTP_COMPRESSION_ENABLED
   if (strcmp(encoding, CURL_COMPRESSION_LITERAL_ALL) == 0)
     self->accept_encoding = g_string_new("");
   else
     self->accept_encoding = g_string_new(encoding);
+#else
+  self->accept_encoding = NULL;
+  msg_warning("http: libcurl has been compiled without compression support. accept-encoding() not supported",
+              evt_tag_str("encoding", encoding));
+#endif
 }
 
-void
-http_dd_set_message_compression(LogDriver *d, const gchar *encoding)
+gboolean
+http_dd_set_content_compression(LogDriver *d, const gchar *encoding)
 {
   HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
 
-  gboolean _encoding_valid = FALSE;
-  _encoding_valid = http_dd_check_curl_compression(encoding);
-  g_assert(_encoding_valid);
-
-  if (http_dd_curl_compression_string_match(encoding, CURL_COMPRESSION_UNCOMPRESSED))
-    {
-      self->message_compression = CURL_COMPRESSION_UNCOMPRESSED;
-      return;
-    }
-  else if (http_dd_curl_compression_string_match(encoding, CURL_COMPRESSION_GZIP))
-    self->message_compression = CURL_COMPRESSION_GZIP;
-  else if (http_dd_curl_compression_string_match(encoding, CURL_COMPRESSION_DEFLATE))
-    self->message_compression = CURL_COMPRESSION_DEFLATE;
-  else
-    self->message_compression = CURL_COMPRESSION_DEFAULT;
+  self->content_compression = compressor_lookup_type(encoding);
+  return self->content_compression != CURL_COMPRESSION_UNKNOWN;
 }
 
+
 void
 http_dd_set_peer_verify(LogDriver *d, gboolean verify)
 {
@@ -400,7 +413,10 @@ http_dd_init(LogPipe *s)
   GlobalConfig *cfg = log_pipe_get_config(s);
 
   if (self->load_balancer->num_targets == 0)
-    http_load_balancer_add_target(self->load_balancer, HTTP_DEFAULT_URL);
+    {
+      GError *error = NULL;
+      g_assert(http_load_balancer_add_target(self->load_balancer, HTTP_DEFAULT_URL, &error));
+    }
 
   if (self->load_balancer->num_targets > 1 && s->persist_name == NULL)
     {
@@ -408,7 +424,7 @@ http_dd_init(LogPipe *s)
                   "It is recommended that you set persist-name() in this case as syslog-ng will be "
                   "using the first URL in urls() to register persistent data, such as the disk queue "
                   "name, which might change",
-                  evt_tag_str("url", self->load_balancer->targets[0].url),
+                  evt_tag_str("url", self->load_balancer->targets[0].url_template->template_str),
                   log_pipe_location_tag(&self->super.super.super.super));
     }
   if (self->load_balancer->num_targets > self->super.num_workers)
@@ -421,11 +437,22 @@ http_dd_init(LogPipe *s)
                   log_pipe_location_tag(&self->super.super.super.super));
     }
   /* we need to set up url before we call the inherited init method, so our stats key is correct */
-  self->url = self->load_balancer->targets[0].url;
+  self->url = self->load_balancer->targets[0].url_template->template_str;
 
   if (!log_threaded_dest_driver_init_method(s))
     return FALSE;
 
+  if ((self->super.batch_lines || self->batch_bytes) && http_load_balancer_is_url_templated(self->load_balancer) &&
+      self->super.num_workers > 1 && !self->super.worker_partition_key)
+    {
+      msg_error("http: worker-partition-key() must be set if using templates in the url() option "
+                "while batching is enabled and multiple workers are configured. "
+                "Make sure to set worker-partition-key() with a template that contains all the templates "
+                "used in the url() option",
+                log_pipe_location_tag(&self->super.super.super.super));
+      return FALSE;
+    }
+
   log_template_options_init(&self->template_options, cfg);
 
   http_load_balancer_set_recovery_timeout(self->load_balancer, self->super.time_reopen);
@@ -483,6 +510,8 @@ http_dd_new(GlobalConfig *cfg)
   self->super.stats_source = stats_register_type("http");
   self->super.worker.construct = http_dw_new;
 
+  log_threaded_dest_driver_set_flush_on_worker_key_change(&self->super.super.super, TRUE);
+
   curl_global_init(CURL_GLOBAL_ALL);
 
   self->ssl_version = CURL_SSLVERSION_DEFAULT;
@@ -501,6 +530,7 @@ http_dd_new(GlobalConfig *cfg)
                                        SYSLOG_NG_VERSION, curl_info->version);
 
   self->response_handlers = http_response_handlers_new();
+  self->content_compression = CURL_COMPRESSION_DEFAULT;
 
   return &self->super.super.super;
 }
