@@ -81,7 +81,7 @@ java_machine_ref(void)
        * therefore the reference counter must be incremented before that.
        * But we are in the _ref() function, so the counter must be updated as below.  */
       g_atomic_counter_inc(&global_jvm->ref_cnt);
-      register_application_hook(AH_SHUTDOWN, java_machine_unref_callback, global_jvm);
+      register_application_hook(AH_SHUTDOWN, java_machine_unref_callback, global_jvm, AHM_RUN_ONCE);
     }
   return global_jvm;
 }
@@ -138,8 +138,11 @@ _is_jvm_option_predefined(const gchar *option)
 {
   static const gchar *predefined_options[] =
   {
+    "Djava.system.class.loader",
+    "Xshare",
     "Djava.class.path",
     "Djava.library.path",
+    "Dlog4j.configurationFactory",
     NULL
   };
 
@@ -186,13 +189,22 @@ _setup_jvm_options_array(JavaVMSingleton *self, const gchar *jvm_options_str)
 {
   GArray *jvm_options_array = _jvm_options_split(jvm_options_str);
 
+  jvm_options_array = _jvm_options_array_append(jvm_options_array,
+                                                g_strdup_printf("-Djava.system.class.loader=org.syslog_ng.SyslogNgClassLoader"));
+
+  jvm_options_array = _jvm_options_array_append(jvm_options_array,
+                                                g_strdup_printf("-Xshare:off"));
+
   jvm_options_array = _jvm_options_array_append(jvm_options_array,
                                                 g_strdup_printf("-Djava.class.path=%s",
                                                     self->class_path->str));
 
   jvm_options_array = _jvm_options_array_append(jvm_options_array,
                                                 g_strdup_printf("-Djava.library.path=%s",
-                                                    resolvedConfigurablePaths.initial_module_path));
+                                                    resolved_configurable_paths.initial_module_path));
+
+  jvm_options_array = _jvm_options_array_append(jvm_options_array,
+                                                g_strdup_printf("-Dlog4j.configurationFactory=org.syslog_ng.logging.CustomConfigurationFactory"));
 
   jvm_options_array = _jvm_options_array_append(jvm_options_array,
                                                 g_strdup("-Xrs"));
