@@ -458,3 +458,19 @@ Test(format_json, test_format_json_with_key_delimiter)
   assert_template_format("$(format-json --key-delimiter ~ top~foo=1 top~bar=2 top~baz=3 top~sub~key1=val1 top~sub~key2=val2)",
                          "{\"top\":{\"sub\":{\"key2\":\"val2\",\"key1\":\"val1\"},\"foo\":\"1\",\"baz\":\"3\",\"bar\":\"2\"}}");
 }
+
+Test(format_json, test_format_json_key_value_with_spaces)
+{
+  assert_template_format("$(format-json foo =alma)",
+                         "{\"foo\":\"alma\"}");
+  assert_template_format("$(format-json foo= alma)",
+                         "{\"foo\":\"alma\"}");
+  assert_template_format("$(format-json foo = alma)",
+                         "{\"foo\":\"alma\"}");
+  assert_template_format("$(format-json foo=\" alma \")",
+                         "{\"foo\":\" alma \"}");
+  assert_template_format("$(format-json foo= \" alma \")",
+                         "{\"foo\":\" alma \"}");
+  assert_template_format("$(format-json foo1= alma foo2 =korte foo3 = szilva foo4 = \" meggy \")",
+                         "{\"foo4\":\" meggy \",\"foo3\":\"szilva\",\"foo2\":\"korte\",\"foo1\":\"alma\"}");
+}
