@@ -32,13 +32,15 @@ typedef struct _PyLogMessage
   PyObject_HEAD
   LogMessage *msg;
   PyObject *bookmark_data;
+  gboolean cast_to_bytes;
 } PyLogMessage;
 
 extern PyTypeObject py_log_message_type;
 
-PyObject *py_log_message_new(LogMessage *msg);
-void py_log_message_init(void);
-
 int py_is_log_message(PyObject *obj);
+PyObject *py_log_message_new(LogMessage *msg, GlobalConfig *cfg);
+
+void py_log_message_global_init(void);
+
 
 #endif
