@@ -44,11 +44,13 @@ setup(void)
 
   cfg = cfg_new_snippet();
   msg_format_options_defaults(&parse_options);
+  msg_format_options_init(&parse_options, cfg);
 }
 
 static void
 teardown(void)
 {
+  msg_format_options_destroy(&parse_options);
   scratch_buffers_explicit_gc();
   app_shutdown();
   cfg_free(cfg);
@@ -62,13 +64,11 @@ Test(syslog_format, parser_should_not_spin_on_non_zero_terminated_input, .timeou
   /* chosen carefully to reproduce a bug */
   gsize data_length = 27;
 
-  msg_format_options_init(&parse_options, cfg);
-  LogMessage *msg = msg_format_construct_message(&parse_options, (const guchar *) data, data_length);
+  LogMessage *msg = log_msg_new_empty();
 
   gsize problem_position;
   cr_assert(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
 
-  msg_format_options_destroy(&parse_options);
   log_msg_unref(msg);
 }
 
@@ -77,14 +77,121 @@ Test(syslog_format, cisco_sequence_id_non_zero_termination)
   const gchar *data = "<189>65536: ";
   gsize data_length = strlen(data);
 
-  msg_format_options_init(&parse_options, cfg);
-  LogMessage *msg = msg_format_construct_message(&parse_options, (const guchar *) data, data_length);
+  LogMessage *msg = log_msg_new_empty();
 
   gsize problem_position;
   cr_assert(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
   cr_assert_str_eq(log_msg_get_value_by_name(msg, ".SDATA.meta.sequenceId", NULL), "65536");
 
-  msg_format_options_destroy(&parse_options);
+  log_msg_unref(msg);
+}
+
+Test(syslog_format, rfc3164_error_invalid_pri)
+{
+  /* incorrect pri value */
+  const gchar *data = "<189 Feb  3 12:34:56 host program[pid]: message";
+  gsize data_length = strlen(data);
+
+  LogMessage *msg = log_msg_new_empty();
+
+  gsize problem_position;
+  cr_assert(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
+  assert_log_message_value_by_name(msg, "MSG", "<189 Feb  3 12:34:56 host program[pid]: message");
+  assert_log_message_has_tag(msg, "syslog.invalid_pri");
+
+  log_msg_unref(msg);
+}
+
+Test(syslog_format, rfc3164_error_missing_timestamp)
+{
+  /* incorrect pri value */
+  const gchar *data = "<189> program[pid]: message";
+  gsize data_length = strlen(data);
+
+  LogMessage *msg = log_msg_new_empty();
+
+  gsize problem_position;
+  cr_assert(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
+  /* without timestamp, host is not expected */
+  assert_log_message_value_by_name(msg, "HOST", "");
+  assert_log_message_value_by_name(msg, "PROGRAM", "program");
+  assert_log_message_value_by_name(msg, "PID", "pid");
+  assert_log_message_value_by_name(msg, "MSG", "message");
+  assert_log_message_value_by_name(msg, "MSGFORMAT", "rfc3164");
+  assert_log_message_has_tag(msg, "syslog.missing_timestamp");
+  assert_log_message_has_tag(msg, "syslog.rfc3164_missing_header");
+
+  log_msg_unref(msg);
+}
+
+Test(syslog_format, rfc5424_error_invalid_timestamp)
+{
+  const gchar *data = "<189>1 2024-09-16Q11:22:33+02:00 host program pid msgid [foo bar=baz] message";
+  gsize data_length = strlen(data);
+
+  parse_options.flags |= LP_SYSLOG_PROTOCOL;
+  LogMessage *msg = log_msg_new_empty();
+
+  gsize problem_position;
+  cr_assert_not(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
+  assert_log_message_value_by_name(msg, "MSGFORMAT", "");
+  assert_log_message_has_tag(msg, "syslog.missing_timestamp");
+
+  log_msg_unref(msg);
+}
+
+Test(syslog_format, rfc5424_error_invalid_sdata)
+{
+  const gchar *data = "<189>1 2024-09-16T11:22:33+02:00 host program pid msgid [foo bar=baz message";
+  gsize data_length = strlen(data);
+
+  parse_options.flags |= LP_SYSLOG_PROTOCOL;
+  LogMessage *msg = log_msg_new_empty();
+
+  gsize problem_position;
+  cr_assert_not(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
+  assert_log_message_value_by_name(msg, "MSGFORMAT", "");
+  assert_log_message_has_tag(msg, "syslog.rfc5424_invalid_sdata");
+
+  log_msg_unref(msg);
+}
+
+Test(syslog_format, rfc3164_style_message_when_parsed_as_rfc5424_is_marked_as_such_in_msgformat)
+{
+  const gchar *data = "<189>Feb  3 12:34:56 host program[pid]: message";
+  gsize data_length = strlen(data);
+
+  parse_options.flags |= LP_SYSLOG_PROTOCOL;
+  LogMessage *msg = log_msg_new_empty();
+
+  gsize problem_position;
+  cr_assert(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
+  assert_log_message_value_by_name(msg, "HOST", "host");
+  assert_log_message_value_by_name(msg, "PROGRAM", "program");
+  assert_log_message_value_by_name(msg, "PID", "pid");
+  assert_log_message_value_by_name(msg, "MSG", "message");
+  assert_log_message_value_by_name(msg, "MSGFORMAT", "rfc3164");
+
+  log_msg_unref(msg);
+}
+
+Test(syslog_format, rfc5424_style_message_when_parsed_as_rfc5424_is_marked_as_such_in_msgformat)
+{
+  const gchar *data = "<189>1 2024-02-03T12:34:56Z host program pid msgid [foo bar=baz] message";
+  gsize data_length = strlen(data);
+
+  parse_options.flags |= LP_SYSLOG_PROTOCOL;
+  LogMessage *msg = log_msg_new_empty();
+
+  gsize problem_position;
+  cr_assert(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
+  assert_log_message_value_by_name(msg, "HOST", "host");
+  assert_log_message_value_by_name(msg, "PROGRAM", "program");
+  assert_log_message_value_by_name(msg, "PID", "pid");
+  assert_log_message_value_by_name(msg, "MSGID", "msgid");
+  assert_log_message_value_by_name(msg, "MSG", "message");
+  assert_log_message_value_by_name(msg, "MSGFORMAT", "rfc5424");
+
   log_msg_unref(msg);
 }
 
@@ -93,14 +200,12 @@ Test(syslog_format, minimal_non_zero_terminated_numeric_message_is_parsed_as_pro
   const gchar *data = "<189>65536";
   gsize data_length = strlen(data);
 
-  msg_format_options_init(&parse_options, cfg);
-  LogMessage *msg = msg_format_construct_message(&parse_options, (const guchar *) data, data_length);
+  LogMessage *msg = log_msg_new_empty();
 
   gsize problem_position;
   cr_assert(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
   cr_assert_str_eq(log_msg_get_value_by_name(msg, "PROGRAM", NULL), "65536");
 
-  msg_format_options_destroy(&parse_options);
   log_msg_unref(msg);
 }
 
@@ -110,12 +215,13 @@ _extract_sdata_into_message_with_prefix(const gchar *sdata, LogMessage **pmsg, c
   const guchar *data = (const guchar *) sdata;
   gint data_length = strlen(sdata);
   LogMessage *msg = log_msg_new_empty();
+  MsgFormatOptions local_parse_options = {0};
 
-  msg_format_options_defaults(&parse_options);
-  msg_format_options_set_sdata_prefix(&parse_options, sdata_prefix);
-  msg_format_options_init(&parse_options, cfg);
-  gboolean result = _syslog_format_parse_sd(msg, &data, &data_length, &parse_options);
-  msg_format_options_destroy(&parse_options);
+  msg_format_options_defaults(&local_parse_options);
+  msg_format_options_set_sdata_prefix(&local_parse_options, sdata_prefix);
+  msg_format_options_init(&local_parse_options, cfg);
+  gboolean result = _syslog_format_parse_sd(msg, &data, &data_length, &local_parse_options);
+  msg_format_options_destroy(&local_parse_options);
 
   if (pmsg)
     *pmsg = msg;
@@ -295,3 +401,48 @@ Test(syslog_format, test_long_sdata)
   strcpy(long_sdata + 1 + long_sdata_id, " a=b]");
   cr_expect_not(_extract_sdata_into_message_with_prefix(long_sdata, NULL, sdata_prefix));
 }
+
+Test(syslog_format, test_frame_checking)
+{
+  const gchar *data = "5 aaaaa";
+  gsize data_length = strlen(data);
+
+  LogMessage *msg = msg_format_construct_message(&parse_options, (const guchar *) data, data_length);
+
+  gsize problem_position;
+  cr_assert(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
+
+  cr_assert(log_msg_is_tag_by_id(msg, LM_T_SYSLOG_UNEXPECTED_FRAMING));
+
+  log_msg_unref(msg);
+}
+
+Test(syslog_format, test_framing_not_detected_when_frame_length_is_too_long)
+{
+  const gchar *data = "99999999999 a";
+  gsize data_length = strlen(data);
+
+  LogMessage *msg = msg_format_construct_message(&parse_options, (const guchar *) data, data_length);
+
+  gsize problem_position;
+  cr_assert(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
+
+  cr_assert_not(log_msg_is_tag_by_id(msg, LM_T_SYSLOG_UNEXPECTED_FRAMING));
+
+  log_msg_unref(msg);
+}
+
+Test(syslog_format, test_framing_not_detected_on_input_starting_with_whitespace)
+{
+  const gchar *data = " non-framed-whitespace";
+  gsize data_length = strlen(data);
+
+  LogMessage *msg = msg_format_construct_message(&parse_options, (const guchar *) data, data_length);
+
+  gsize problem_position;
+  cr_assert(syslog_format_handler(&parse_options, msg, (const guchar *) data, data_length, &problem_position));
+
+  cr_assert_not(log_msg_is_tag_by_id(msg, LM_T_SYSLOG_UNEXPECTED_FRAMING));
+
+  log_msg_unref(msg);
+}
