@@ -27,7 +27,9 @@ from messagecheck import *
 from control import flush_files, stop_syslogng
 import os
 
-config = """@version: 3.25
+config = """@version: %(syslog_ng_version)s
+
+@module mod-python use-virtualenv(no)
 
 options { keep-hostname(yes); };
 
@@ -43,7 +45,9 @@ log { source(s_tcp); destination(d_python); };
 
 python {
 
-class MyParser(object):
+from syslogng import LogParser
+
+class MyParser(LogParser):
   def init(self, options):
       return True
 
@@ -73,15 +77,15 @@ def get_source_dir():
 def check_env():
 
     if not has_module('mod-python'):
-        print 'Python module is not available, skipping Python test'
+        print('Python module is not available, skipping Python test')
         return False
 
     src_dir = get_source_dir()
-    print src_dir
+    print(src_dir)
     if 'PYTHONPATH' not in os.environ or src_dir not in os.environ['PYTHONPATH']:
         os.environ['PYTHONPATH'] = os.environ.get('PYTHONPATH', '') + ':' + src_dir
 
-    print 'Python module found, proceeding to Python tests'
+    print('Python module found, proceeding to Python tests')
     return True
 
 
