@@ -80,7 +80,6 @@ class SyslogNgCli(object):
     def __wait_for_control_socket_alive(self):
         def is_alive(s):
             if not s.is_process_running():
-                self.__process = None
                 self.__error_handling("syslog-ng is not running")
             return s.__syslog_ng_ctl.is_control_socket_alive()
         return wait_until_true(is_alive, self)
@@ -167,6 +166,10 @@ class SyslogNgCli(object):
                 core_file.replace(Path(core_file))
             if core_file_found:
                 raise Exception("syslog-ng core file was found and processed")
+            if self.__process.returncode in [-6, -9, -11]:
+                ret_code = self.__process.returncode
+                self.__process = None
+                raise Exception("syslog-ng process crashed with signal {}".format(ret_code))
 
     def set_start_parameters(self, stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path):
         self.__stderr = stderr
