@@ -62,6 +62,10 @@ typedef struct _thread_data
   struct timeval last_throttle_check;
   long buckets;
   gboolean proxy_header_sent;
+
+  /* timestamp  cache for logline generator */
+  struct timeval ts_formatted;
+  char stamp[32];
 } ThreadData;
 
 typedef GOptionEntry *(*get_option_func)(void);
